/* Meiga - Lightweight and easy to use web file server for your desktop
 *
 * Copyright (C) 2009 Igalia, S.L.
 *
 * Authors:
 *
 * Igalia, S.L. <info@igalia.com>
 * Enrique Ocaña González <eocanha@igalia.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>


#define TYPE_LOG (log_get_type ())
#define LOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LOG, Log))
#define LOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LOG, LogClass))
#define IS_LOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LOG))
#define IS_LOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LOG))
#define LOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LOG, LogClass))

typedef struct _Log Log;
typedef struct _LogClass LogClass;
typedef struct _LogPrivate LogPrivate;

#define TYPE_LOG_TASK (log_task_get_type ())
#define LOG_TASK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LOG_TASK, LogTask))
#define LOG_TASK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LOG_TASK, LogTaskClass))
#define IS_LOG_TASK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LOG_TASK))
#define IS_LOG_TASK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LOG_TASK))
#define LOG_TASK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LOG_TASK, LogTaskClass))

typedef struct _LogTask LogTask;
typedef struct _LogTaskClass LogTaskClass;
typedef struct _LogTaskPrivate LogTaskPrivate;

struct _Log {
	GObject parent_instance;
	LogPrivate * priv;
};

struct _LogClass {
	GObjectClass parent_class;
};

struct _LogPrivate {
	GPtrArray* lines;
	guint history_offset;
	guint purge_trigger;
	guint _history_limit;
	guint _history_hysteresis;
};

struct _LogTask {
	GObject parent_instance;
	LogTaskPrivate * priv;
	char* msg;
	Log* logger;
};

struct _LogTaskClass {
	GObjectClass parent_class;
};


static gpointer log_parent_class = NULL;
static gpointer log_task_parent_class = NULL;

GType log_get_type (void);
#define LOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_LOG, LogPrivate))
enum  {
	LOG_DUMMY_PROPERTY,
	LOG_HISTORY_LIMIT,
	LOG_HISTORY_HYSTERESIS
};
Log* log_new (void);
Log* log_construct (GType object_type);
LogTask* log_task_new (const char* msg, Log* logger);
LogTask* log_task_construct (GType object_type, const char* msg, Log* logger);
GType log_task_get_type (void);
void log_task_idle_add (LogTask* self);
void log_log (Log* self, const char* msg);
guint log_get_history_hysteresis (Log* self);
guint log_get_history_limit (Log* self);
void log_log_impl (Log* self, const char* msg);
char* log_get_pending (Log* self, guint start);
void log_set_history_limit (Log* self, guint value);
void log_set_history_hysteresis (Log* self, guint value);
static void log_finalize (GObject* obj);
static void log_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void log_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
enum  {
	LOG_TASK_DUMMY_PROPERTY
};
gboolean log_task_log_on_idle (LogTask* self);
static gboolean _log_task_log_on_idle_gsource_func (gpointer self);
static void log_task_finalize (GObject* obj);



Log* log_construct (GType object_type) {
	Log * self;
	GPtrArray* _tmp0_;
	self = g_object_newv (object_type, 0, NULL);
	_tmp0_ = NULL;
	self->priv->lines = (_tmp0_ = g_ptr_array_new (), (self->priv->lines == NULL) ? NULL : (self->priv->lines = (g_ptr_array_free (self->priv->lines, TRUE), NULL)), _tmp0_);
	self->priv->history_offset = (guint) 0;
	self->priv->purge_trigger = (guint) 0;
	return self;
}


Log* log_new (void) {
	return log_construct (TYPE_LOG);
}


/** Logs a line of text (thread safe) */
void log_log (Log* self, const char* msg) {
	LogTask* t;
	g_return_if_fail (self != NULL);
	g_return_if_fail (msg != NULL);
	t = log_task_new (msg, self);
	log_task_idle_add (t);
	(t == NULL) ? NULL : (t = (g_object_unref (t), NULL));
}


/** Thread unsafe implementation */
void log_log_impl (Log* self, const char* msg) {
	const char* s;
	gboolean _tmp0_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (msg != NULL);
	/* Workaround for memory management problems*/
	s = g_strdup_printf ("%s", msg);
	g_ptr_array_add (self->priv->lines, s);
	self->priv->purge_trigger++;
	_tmp0_ = FALSE;
	if (self->priv->purge_trigger > self->priv->_history_hysteresis) {
		_tmp0_ = self->priv->lines->len > self->priv->_history_limit;
	} else {
		_tmp0_ = FALSE;
	}
	/* If the array grows larger than the history_limit, forget the
	 first items. We only check this every "history_hysteresis"
	 insertions.*/
	if (_tmp0_) {
		self->priv->history_offset = self->priv->history_offset + (self->priv->lines->len - self->priv->_history_limit);
		{
			guint i;
			/* Workaround for memory management problems. Memory freeing*/
			i = (guint) 0;
			{
				gboolean _tmp1_;
				_tmp1_ = TRUE;
				while (TRUE) {
					if (!_tmp1_) {
						i++;
					}
					_tmp1_ = FALSE;
					if (!(i < (self->priv->lines->len - self->priv->_history_limit))) {
						break;
					}
					g_free (self->priv->lines->pdata[i]);
				}
			}
		}
		g_ptr_array_remove_range (self->priv->lines, (guint) 0, self->priv->lines->len - self->priv->_history_limit);
		self->priv->purge_trigger = (guint) 0;
	}
	/* Notify observers*/
	g_signal_emit_by_name (self, "changed");
}


/**
   * Gets stored log lines from the start index to the end of the log.
   * Some lines may have been lost. In that case the set returned will
   * start in the first recorded line.
   */
char* log_get_pending (Log* self, guint start) {
	char* result;
	guint _tmp0_;
	guint i0;
	GString* _result_;
	const char* _tmp2_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = 0U;
	if (start < self->priv->history_offset) {
		_tmp0_ = self->priv->history_offset;
	} else {
		_tmp0_ = start;
	}
	i0 = _tmp0_;
	_result_ = g_string_new ("");
	{
		guint i;
		i = i0 - self->priv->history_offset;
		{
			gboolean _tmp1_;
			_tmp1_ = TRUE;
			while (TRUE) {
				const char* line;
				if (!_tmp1_) {
					i++;
				}
				_tmp1_ = FALSE;
				if (!(i < self->priv->lines->len)) {
					break;
				}
				line = self->priv->lines->pdata[i];
				g_string_append (_result_, line);
				g_string_append (_result_, "\n");
			}
		}
	}
	fprintf (stderr, _result_->str);
	_tmp2_ = NULL;
	result = (_tmp2_ = _result_->str, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_));
	(_result_ == NULL) ? NULL : (_result_ = (g_string_free (_result_, TRUE), NULL));
	return result;
	(_result_ == NULL) ? NULL : (_result_ = (g_string_free (_result_, TRUE), NULL));
}


guint log_get_history_limit (Log* self) {
	guint result;
	g_return_val_if_fail (self != NULL, 0U);
	result = self->priv->_history_limit;
	return result;
}


void log_set_history_limit (Log* self, guint value) {
	g_return_if_fail (self != NULL);
	self->priv->_history_limit = value;
	g_object_notify ((GObject *) self, "history-limit");
}


guint log_get_history_hysteresis (Log* self) {
	guint result;
	g_return_val_if_fail (self != NULL, 0U);
	result = self->priv->_history_hysteresis;
	return result;
}


void log_set_history_hysteresis (Log* self, guint value) {
	g_return_if_fail (self != NULL);
	self->priv->_history_hysteresis = value;
	g_object_notify ((GObject *) self, "history-hysteresis");
}


static void log_class_init (LogClass * klass) {
	log_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (LogPrivate));
	G_OBJECT_CLASS (klass)->get_property = log_get_property;
	G_OBJECT_CLASS (klass)->set_property = log_set_property;
	G_OBJECT_CLASS (klass)->finalize = log_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), LOG_HISTORY_LIMIT, g_param_spec_uint ("history-limit", "history-limit", "history-limit", 0, G_MAXUINT, (guint) 1024, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), LOG_HISTORY_HYSTERESIS, g_param_spec_uint ("history-hysteresis", "history-hysteresis", "history-hysteresis", 0, G_MAXUINT, (guint) 32, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_signal_new ("changed", TYPE_LOG, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void log_instance_init (Log * self) {
	self->priv = LOG_GET_PRIVATE (self);
	self->priv->_history_limit = (guint) 1024;
	self->priv->_history_hysteresis = (guint) 32;
}


static void log_finalize (GObject* obj) {
	Log * self;
	self = LOG (obj);
	(self->priv->lines == NULL) ? NULL : (self->priv->lines = (g_ptr_array_free (self->priv->lines, TRUE), NULL));
	G_OBJECT_CLASS (log_parent_class)->finalize (obj);
}


GType log_get_type (void) {
	static GType log_type_id = 0;
	if (log_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (LogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) log_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Log), 0, (GInstanceInitFunc) log_instance_init, NULL };
		log_type_id = g_type_register_static (G_TYPE_OBJECT, "Log", &g_define_type_info, 0);
	}
	return log_type_id;
}


static void log_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Log * self;
	gpointer boxed;
	self = LOG (object);
	switch (property_id) {
		case LOG_HISTORY_LIMIT:
		g_value_set_uint (value, log_get_history_limit (self));
		break;
		case LOG_HISTORY_HYSTERESIS:
		g_value_set_uint (value, log_get_history_hysteresis (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void log_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Log * self;
	self = LOG (object);
	switch (property_id) {
		case LOG_HISTORY_LIMIT:
		log_set_history_limit (self, g_value_get_uint (value));
		break;
		case LOG_HISTORY_HYSTERESIS:
		log_set_history_hysteresis (self, g_value_get_uint (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


LogTask* log_task_construct (GType object_type, const char* msg, Log* logger) {
	LogTask * self;
	char* _tmp1_;
	const char* _tmp0_;
	Log* _tmp3_;
	Log* _tmp2_;
	g_return_val_if_fail (msg != NULL, NULL);
	g_return_val_if_fail (logger != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->msg = (_tmp1_ = (_tmp0_ = msg, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->msg = (g_free (self->msg), NULL), _tmp1_);
	_tmp3_ = NULL;
	_tmp2_ = NULL;
	self->logger = (_tmp3_ = (_tmp2_ = logger, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (self->logger == NULL) ? NULL : (self->logger = (g_object_unref (self->logger), NULL)), _tmp3_);
	return self;
}


LogTask* log_task_new (const char* msg, Log* logger) {
	return log_task_construct (TYPE_LOG_TASK, msg, logger);
}


static gboolean _log_task_log_on_idle_gsource_func (gpointer self) {
	return log_task_log_on_idle (self);
}


void log_task_idle_add (LogTask* self) {
	g_return_if_fail (self != NULL);
	g_object_ref ((GObject*) self);
	g_idle_add (_log_task_log_on_idle_gsource_func, self);
}


gboolean log_task_log_on_idle (LogTask* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	log_log_impl (self->logger, self->msg);
	g_object_unref ((GObject*) self);
	result = FALSE;
	return result;
}


static void log_task_class_init (LogTaskClass * klass) {
	log_task_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = log_task_finalize;
}


static void log_task_instance_init (LogTask * self) {
}


static void log_task_finalize (GObject* obj) {
	LogTask * self;
	self = LOG_TASK (obj);
	self->msg = (g_free (self->msg), NULL);
	(self->logger == NULL) ? NULL : (self->logger = (g_object_unref (self->logger), NULL));
	G_OBJECT_CLASS (log_task_parent_class)->finalize (obj);
}


GType log_task_get_type (void) {
	static GType log_task_type_id = 0;
	if (log_task_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (LogTaskClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) log_task_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (LogTask), 0, (GInstanceInitFunc) log_task_instance_init, NULL };
		log_task_type_id = g_type_register_static (G_TYPE_OBJECT, "LogTask", &g_define_type_info, 0);
	}
	return log_task_type_id;
}




