/* Meiga - Lightweight and easy to use web file server for your desktop
 *
 * Copyright (C) 2009 Igalia, S.L.
 *
 * Authors:
 *
 * Igalia, S.L. <info@igalia.com>
 * Enrique Ocaña González <eocanha@igalia.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>


#define TYPE_NET (net_get_type ())
#define NET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_NET, Net))
#define NET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_NET, NetClass))
#define IS_NET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_NET))
#define IS_NET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_NET))
#define NET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_NET, NetClass))

typedef struct _Net Net;
typedef struct _NetClass NetClass;
typedef struct _NetPrivate NetPrivate;

#define TYPE_LOG (log_get_type ())
#define LOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LOG, Log))
#define LOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LOG, LogClass))
#define IS_LOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LOG))
#define IS_LOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LOG))
#define LOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LOG, LogClass))

typedef struct _Log Log;
typedef struct _LogClass LogClass;

struct _Net {
	GObject parent_instance;
	NetPrivate * priv;
};

struct _NetClass {
	GObjectClass parent_class;
};

struct _NetPrivate {
	GThread* worker;
	GStaticRecMutex __lock_worker;
	char* _external_ip;
	char* _internal_ip;
	gint _port;
	char* _url;
	gint _previous_redirection_type;
	gint _redirection_type;
	gint _redirection_status;
	char* _ssh_host;
	char* _ssh_port;
	char* _ssh_user;
	char* _ssh_password;
	char* _display;
	Log* _logger;
};


static gpointer net_parent_class = NULL;

GType net_get_type (void);
GType log_get_type (void);
#define NET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_NET, NetPrivate))
enum  {
	NET_DUMMY_PROPERTY,
	NET_PORT,
	NET_URL,
	NET_REDIRECTION_TYPE,
	NET_REDIRECTION_STATUS,
	NET_SSH_HOST,
	NET_SSH_PORT,
	NET_SSH_USER,
	NET_SSH_PASSWORD,
	NET_DISPLAY,
	NET_LOGGER
};
#define NET_REDIRECTION_TYPE_NONE 0
#define NET_REDIRECTION_TYPE_UPNP 1
#define NET_REDIRECTION_TYPE_SSH 2
#define NET_REDIRECTION_STATUS_NONE 0
#define NET_REDIRECTION_STATUS_PENDING 1
#define NET_REDIRECTION_STATUS_DONE 2
#define NET_REDIRECTION_STATUS_ERROR 3
void net_set_port (Net* self, gint value);
static void net_set_url (Net* self, const char* value);
void net_set_logger (Net* self, Log* value);
void net_set_ssh_host (Net* self, const char* value);
void net_set_ssh_user (Net* self, const char* value);
void net_set_display (Net* self, const char* value);
static void* net_forward_none_start (Net* self);
Net* net_new (void);
Net* net_construct (GType object_type);
void log_log (Log* self, const char* msg);
static void net_log (Net* self, const char* msg);
static void net_forward_stop (Net* self);
static void net_forward_start (Net* self);
gboolean net_forward_reload (Net* self);
gint net_get_redirection_type (Net* self);
static void* net_forward_upnp_start (Net* self);
static void* _net_forward_upnp_start_gthread_func (gpointer self);
static void* net_forward_ssh_start (Net* self);
static void* _net_forward_ssh_start_gthread_func (gpointer self);
void net_forward_upnp_stop (Net* self);
void net_forward_ssh_stop (Net* self);
gint net_get_port (Net* self);
static void net_set_internal_ip (Net* self, const char* value);
static void net_set_external_ip (Net* self, const char* value);
void net_set_redirection_status (Net* self, gint value);
static gboolean _lambda5_ (Net* self);
static gboolean __lambda5__gsource_func (gpointer self);
static gboolean _lambda6_ (Net* self);
static gboolean __lambda6__gsource_func (gpointer self);
gint net_get_redirection_status (Net* self);
static char* net_get_internal_ip (Net* self);
static char* net_get_external_ip (Net* self);
static gboolean _lambda3_ (Net* self);
static gboolean __lambda3__gsource_func (gpointer self);
static gboolean _lambda7_ (Net* self);
static gboolean __lambda7__gsource_func (gpointer self);
static gboolean _lambda8_ (Net* self);
static gboolean __lambda8__gsource_func (gpointer self);
char* net_get_ssh_host (Net* self);
char* net_get_ssh_user (Net* self);
char* net_get_ssh_password (Net* self);
char* net_get_ssh_port (Net* self);
char* net_get_display (Net* self);
static gboolean _lambda4_ (Net* self);
static gboolean __lambda4__gsource_func (gpointer self);
char* net_get_url (Net* self);
static gboolean _net_forward_reload_gsource_func (gpointer self);
void net_set_redirection_type (Net* self, gint value);
void net_set_ssh_port (Net* self, const char* value);
void net_set_ssh_password (Net* self, const char* value);
static Log* net_get_logger (Net* self);
static void net_finalize (GObject* obj);
static void net_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void net_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



Net* net_construct (GType object_type) {
	Net * self;
	self = g_object_newv (object_type, 0, NULL);
	net_set_port (self, 8001);
	net_set_url (self, NULL);
	net_set_logger (self, NULL);
	self->priv->_previous_redirection_type = NET_REDIRECTION_TYPE_NONE;
	self->priv->_redirection_type = NET_REDIRECTION_TYPE_NONE;
	self->priv->_redirection_status = NET_REDIRECTION_STATUS_NONE;
	net_set_ssh_host (self, "");
	net_set_ssh_user (self, "");
	net_set_display (self, "");
	net_forward_none_start (self);
	return self;
}


Net* net_new (void) {
	return net_construct (TYPE_NET);
}


static void net_log (Net* self, const char* msg) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (msg != NULL);
	if (self->priv->_logger != NULL) {
		log_log (self->priv->_logger, msg);
	}
}


gboolean net_forward_reload (Net* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	if (self->priv->_redirection_type != self->priv->_previous_redirection_type) {
		net_forward_stop (self);
		net_forward_start (self);
	}
	result = FALSE;
	return result;
}


static void* _net_forward_upnp_start_gthread_func (gpointer self) {
	return net_forward_upnp_start (self);
}


static void* _net_forward_ssh_start_gthread_func (gpointer self) {
	return net_forward_ssh_start (self);
}


static void net_forward_start (Net* self) {
	GError * _inner_error_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	switch (net_get_redirection_type (self)) {
		case NET_REDIRECTION_TYPE_NONE:
		{
			net_forward_none_start (self);
			break;
		}
		case NET_REDIRECTION_TYPE_UPNP:
		{
			{
				GThread* _tmp0_;
				_tmp0_ = g_thread_create (_net_forward_upnp_start_gthread_func, self, TRUE, &_inner_error_);
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == G_SPAWN_ERROR) {
						goto __catch6_g_spawn_error;
					}
					if (_inner_error_->domain == G_THREAD_ERROR) {
						goto __catch6_g_thread_error;
					}
					goto __finally6;
				}
				self->priv->worker = _tmp0_;
			}
			goto __finally6;
			__catch6_g_spawn_error:
			{
				GError * e;
				e = _inner_error_;
				_inner_error_ = NULL;
				{
					net_log (self, _ ("Error spawning UPNP redirector process"));
					(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
				}
			}
			goto __finally6;
			__catch6_g_thread_error:
			{
				GError * e;
				e = _inner_error_;
				_inner_error_ = NULL;
				{
					net_log (self, _ ("Error creating thread for UPNP redirector process"));
					(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
				}
			}
			__finally6:
			if (_inner_error_ != NULL) {
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
			break;
		}
		case NET_REDIRECTION_TYPE_SSH:
		{
			{
				GThread* _tmp1_;
				_tmp1_ = g_thread_create (_net_forward_ssh_start_gthread_func, self, TRUE, &_inner_error_);
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == G_SPAWN_ERROR) {
						goto __catch7_g_spawn_error;
					}
					if (_inner_error_->domain == G_THREAD_ERROR) {
						goto __catch7_g_thread_error;
					}
					goto __finally7;
				}
				self->priv->worker = _tmp1_;
			}
			goto __finally7;
			__catch7_g_spawn_error:
			{
				GError * e;
				e = _inner_error_;
				_inner_error_ = NULL;
				{
					net_log (self, _ ("Error spawning UPNP redirector process"));
					(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
				}
			}
			goto __finally7;
			__catch7_g_thread_error:
			{
				GError * e;
				e = _inner_error_;
				_inner_error_ = NULL;
				{
					net_log (self, _ ("Error creating thread for UPNP redirector process"));
					(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
				}
			}
			__finally7:
			if (_inner_error_ != NULL) {
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
			break;
		}
	}
}


static void net_forward_stop (Net* self) {
	g_return_if_fail (self != NULL);
	switch (self->priv->_previous_redirection_type) {
		case NET_REDIRECTION_TYPE_UPNP:
		{
			net_forward_upnp_stop (self);
			break;
		}
		case NET_REDIRECTION_TYPE_SSH:
		{
			net_forward_ssh_stop (self);
			break;
		}
	}
}


static void* net_forward_none_start (Net* self) {
	void* result;
	GError * _inner_error_;
	char* internal_ip;
	char* url;
	char* txtout;
	char* txterr;
	gint _result_;
	char* _tmp5_;
	gboolean _tmp4_;
	char* _tmp3_;
	char* _tmp2_;
	gboolean _tmp1_;
	char* _tmp0_;
	char* _tmp10_;
	g_return_val_if_fail (self != NULL, NULL);
	_inner_error_ = NULL;
	internal_ip = NULL;
	url = NULL;
	txtout = NULL;
	txterr = NULL;
	_result_ = 0;
	_tmp5_ = NULL;
	_tmp3_ = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	_tmp4_ = (_tmp1_ = g_spawn_command_line_sync (BINDIR "/fwlocalip", &_tmp0_, &_tmp3_, &_result_, &_inner_error_), txtout = (_tmp2_ = _tmp0_, txtout = (g_free (txtout), NULL), _tmp2_), _tmp1_);
	txterr = (_tmp5_ = _tmp3_, txterr = (g_free (txterr), NULL), _tmp5_);
	_tmp4_;
	if (_inner_error_ != NULL) {
		internal_ip = (g_free (internal_ip), NULL);
		url = (g_free (url), NULL);
		txtout = (g_free (txtout), NULL);
		txterr = (g_free (txterr), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	if (_result_ == 0) {
		char* _tmp7_;
		const char* _tmp6_;
		char* _tmp8_;
		_tmp7_ = NULL;
		_tmp6_ = NULL;
		internal_ip = (_tmp7_ = (_tmp6_ = g_strchomp (txtout), (_tmp6_ == NULL) ? NULL : g_strdup (_tmp6_)), internal_ip = (g_free (internal_ip), NULL), _tmp7_);
		_tmp8_ = NULL;
		net_log (self, _tmp8_ = g_strdup_printf (_ ("Found internal IP: %s"), internal_ip));
		_tmp8_ = (g_free (_tmp8_), NULL);
	} else {
		char* _tmp9_;
		net_log (self, _ ("Local IP not found"));
		_tmp9_ = NULL;
		internal_ip = (_tmp9_ = g_strdup ("127.0.0.1"), internal_ip = (g_free (internal_ip), NULL), _tmp9_);
	}
	_tmp10_ = NULL;
	url = (_tmp10_ = g_strdup_printf ("http://%s:%d", internal_ip, net_get_port (self)), url = (g_free (url), NULL), _tmp10_);
	net_set_internal_ip (self, internal_ip);
	net_set_external_ip (self, internal_ip);
	net_set_url (self, url);
	net_set_redirection_status (self, NET_REDIRECTION_STATUS_NONE);
	result = NULL;
	internal_ip = (g_free (internal_ip), NULL);
	url = (g_free (url), NULL);
	txtout = (g_free (txtout), NULL);
	txterr = (g_free (txterr), NULL);
	return result;
}


static gboolean _lambda5_ (Net* self) {
	gboolean result;
	net_set_redirection_status (self, NET_REDIRECTION_STATUS_PENDING);
	result = FALSE;
	return result;
}


static gboolean __lambda5__gsource_func (gpointer self) {
	return _lambda5_ (self);
}


static gboolean _lambda6_ (Net* self) {
	gboolean result;
	net_set_redirection_status (self, NET_REDIRECTION_STATUS_ERROR);
	result = FALSE;
	return result;
}


static gboolean __lambda6__gsource_func (gpointer self) {
	return _lambda6_ (self);
}


static void* net_forward_upnp_start (Net* self) {
	void* result;
	GError * _inner_error_;
	char* internal_ip;
	char* external_ip;
	char* url;
	char* txtout;
	char* txterr;
	gint _result_;
	gint status;
	char* _tmp5_;
	gboolean _tmp4_;
	char* _tmp3_;
	char* _tmp2_;
	gboolean _tmp1_;
	char* _tmp0_;
	char* _tmp11_;
	const char* _tmp10_;
	char* _tmp41_;
	char* _tmp43_;
	const char* _tmp42_;
	char* _tmp45_;
	const char* _tmp44_;
	char* _tmp47_;
	const char* _tmp46_;
	g_return_val_if_fail (self != NULL, NULL);
	_inner_error_ = NULL;
	internal_ip = NULL;
	external_ip = NULL;
	url = NULL;
	txtout = NULL;
	txterr = NULL;
	_result_ = 0;
	status = 0;
	status = NET_REDIRECTION_STATUS_PENDING;
	g_idle_add (__lambda5__gsource_func, self);
	_tmp5_ = NULL;
	_tmp3_ = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	_tmp4_ = (_tmp1_ = g_spawn_command_line_sync (BINDIR "/fwlocalip", &_tmp0_, &_tmp3_, &_result_, &_inner_error_), txtout = (_tmp2_ = _tmp0_, txtout = (g_free (txtout), NULL), _tmp2_), _tmp1_);
	txterr = (_tmp5_ = _tmp3_, txterr = (g_free (txterr), NULL), _tmp5_);
	_tmp4_;
	if (_inner_error_ != NULL) {
		internal_ip = (g_free (internal_ip), NULL);
		external_ip = (g_free (external_ip), NULL);
		url = (g_free (url), NULL);
		txtout = (g_free (txtout), NULL);
		txterr = (g_free (txterr), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	if (_result_ == 0) {
		char* _tmp7_;
		const char* _tmp6_;
		char* _tmp8_;
		_tmp7_ = NULL;
		_tmp6_ = NULL;
		internal_ip = (_tmp7_ = (_tmp6_ = g_strchomp (txtout), (_tmp6_ == NULL) ? NULL : g_strdup (_tmp6_)), internal_ip = (g_free (internal_ip), NULL), _tmp7_);
		_tmp8_ = NULL;
		net_log (self, _tmp8_ = g_strdup_printf (_ ("Found internal IP: %s"), internal_ip));
		_tmp8_ = (g_free (_tmp8_), NULL);
	} else {
		char* _tmp9_;
		net_log (self, _ ("Local IP not found"));
		_tmp9_ = NULL;
		internal_ip = (_tmp9_ = g_strdup ("127.0.0.1"), internal_ip = (g_free (internal_ip), NULL), _tmp9_);
		g_idle_add (__lambda6__gsource_func, self);
		result = NULL;
		internal_ip = (g_free (internal_ip), NULL);
		external_ip = (g_free (external_ip), NULL);
		url = (g_free (url), NULL);
		txtout = (g_free (txtout), NULL);
		txterr = (g_free (txterr), NULL);
		return result;
	}
	_tmp11_ = NULL;
	_tmp10_ = NULL;
	external_ip = (_tmp11_ = (_tmp10_ = internal_ip, (_tmp10_ == NULL) ? NULL : g_strdup (_tmp10_)), external_ip = (g_free (external_ip), NULL), _tmp11_);
	/* Don't redirect if there's no valid internal IP*/
	if (strcmp (internal_ip, "127.0.0.1") != 0) {
		char* _tmp17_;
		gboolean _tmp16_;
		char* _tmp15_;
		char* _tmp14_;
		gboolean _tmp13_;
		char* _tmp12_;
		char* _tmp19_;
		const char* _tmp18_;
		gboolean _tmp20_;
		gboolean _tmp21_;
		_tmp17_ = NULL;
		_tmp15_ = NULL;
		_tmp14_ = NULL;
		_tmp12_ = NULL;
		_tmp16_ = (_tmp13_ = g_spawn_command_line_sync (BINDIR "/fwupnp -i", &_tmp12_, &_tmp15_, &_result_, &_inner_error_), txtout = (_tmp14_ = _tmp12_, txtout = (g_free (txtout), NULL), _tmp14_), _tmp13_);
		txterr = (_tmp17_ = _tmp15_, txterr = (g_free (txterr), NULL), _tmp17_);
		_tmp16_;
		if (_inner_error_ != NULL) {
			internal_ip = (g_free (internal_ip), NULL);
			external_ip = (g_free (external_ip), NULL);
			url = (g_free (url), NULL);
			txtout = (g_free (txtout), NULL);
			txterr = (g_free (txterr), NULL);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return NULL;
		}
		_tmp19_ = NULL;
		_tmp18_ = NULL;
		txtout = (_tmp19_ = (_tmp18_ = g_strchomp (txtout), (_tmp18_ == NULL) ? NULL : g_strdup (_tmp18_)), txtout = (g_free (txtout), NULL), _tmp19_);
		_tmp20_ = FALSE;
		_tmp21_ = FALSE;
		if (_result_ == 0) {
			_tmp21_ = strcmp (txtout, "") != 0;
		} else {
			_tmp21_ = FALSE;
		}
		if (_tmp21_) {
			_tmp20_ = strcmp (txtout, "(null)") != 0;
		} else {
			_tmp20_ = FALSE;
		}
		if (_tmp20_) {
			char* _tmp23_;
			const char* _tmp22_;
			char* _tmp24_;
			char* _tmp32_;
			gboolean _tmp31_;
			char* _tmp30_;
			char* _tmp29_;
			gboolean _tmp28_;
			char* _tmp27_;
			char* _tmp26_;
			char* _tmp25_;
			_tmp23_ = NULL;
			_tmp22_ = NULL;
			external_ip = (_tmp23_ = (_tmp22_ = txtout, (_tmp22_ == NULL) ? NULL : g_strdup (_tmp22_)), external_ip = (g_free (external_ip), NULL), _tmp23_);
			_tmp24_ = NULL;
			net_log (self, _tmp24_ = g_strdup_printf (_ ("Found external IP: %s"), external_ip));
			_tmp24_ = (g_free (_tmp24_), NULL);
			_tmp32_ = NULL;
			_tmp30_ = NULL;
			_tmp29_ = NULL;
			_tmp27_ = NULL;
			_tmp26_ = NULL;
			_tmp25_ = NULL;
			_tmp31_ = (_tmp28_ = g_spawn_command_line_sync (_tmp26_ = g_strconcat (BINDIR, _tmp25_ = g_strdup_printf ("/fwupnp -q %d", net_get_port (self)), NULL), &_tmp27_, &_tmp30_, &_result_, &_inner_error_), txtout = (_tmp29_ = _tmp27_, txtout = (g_free (txtout), NULL), _tmp29_), _tmp28_);
			txterr = (_tmp32_ = _tmp30_, txterr = (g_free (txterr), NULL), _tmp32_);
			_tmp31_;
			if (_inner_error_ != NULL) {
				internal_ip = (g_free (internal_ip), NULL);
				external_ip = (g_free (external_ip), NULL);
				url = (g_free (url), NULL);
				txtout = (g_free (txtout), NULL);
				txterr = (g_free (txterr), NULL);
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return NULL;
			}
			_tmp26_ = (g_free (_tmp26_), NULL);
			_tmp25_ = (g_free (_tmp25_), NULL);
			if (_result_ != 0) {
				char* _tmp40_;
				gboolean _tmp39_;
				char* _tmp38_;
				char* _tmp37_;
				gboolean _tmp36_;
				char* _tmp35_;
				char* _tmp34_;
				char* _tmp33_;
				net_log (self, _ ("Creating redirection"));
				_tmp40_ = NULL;
				_tmp38_ = NULL;
				_tmp37_ = NULL;
				_tmp35_ = NULL;
				_tmp34_ = NULL;
				_tmp33_ = NULL;
				_tmp39_ = (_tmp36_ = g_spawn_command_line_sync (_tmp34_ = g_strconcat (BINDIR, _tmp33_ = g_strdup_printf ("/fwupnp -r %d %d %s %s %d", net_get_port (self), net_get_port (self), internal_ip, "Meiga", 0), NULL), &_tmp35_, &_tmp38_, &_result_, &_inner_error_), txtout = (_tmp37_ = _tmp35_, txtout = (g_free (txtout), NULL), _tmp37_), _tmp36_);
				txterr = (_tmp40_ = _tmp38_, txterr = (g_free (txterr), NULL), _tmp40_);
				_tmp39_;
				if (_inner_error_ != NULL) {
					internal_ip = (g_free (internal_ip), NULL);
					external_ip = (g_free (external_ip), NULL);
					url = (g_free (url), NULL);
					txtout = (g_free (txtout), NULL);
					txterr = (g_free (txterr), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return NULL;
				}
				_tmp34_ = (g_free (_tmp34_), NULL);
				_tmp33_ = (g_free (_tmp33_), NULL);
				if (_result_ == 0) {
					net_log (self, _ ("Redirection performed"));
					status = NET_REDIRECTION_STATUS_DONE;
				}
			} else {
				net_log (self, _ ("Redirection already present"));
				status = NET_REDIRECTION_STATUS_DONE;
			}
		} else {
			net_log (self, _ ("External IP not found. Check that your router has UPnP enabled and working"));
			status = NET_REDIRECTION_STATUS_ERROR;
		}
	}
	_tmp41_ = NULL;
	url = (_tmp41_ = g_strdup_printf ("http://%s:%d", external_ip, net_get_port (self)), url = (g_free (url), NULL), _tmp41_);
	_tmp43_ = NULL;
	_tmp42_ = NULL;
	internal_ip = (_tmp43_ = (_tmp42_ = internal_ip, (_tmp42_ == NULL) ? NULL : g_strdup (_tmp42_)), internal_ip = (g_free (internal_ip), NULL), _tmp43_);
	_tmp45_ = NULL;
	_tmp44_ = NULL;
	external_ip = (_tmp45_ = (_tmp44_ = external_ip, (_tmp44_ == NULL) ? NULL : g_strdup (_tmp44_)), external_ip = (g_free (external_ip), NULL), _tmp45_);
	_tmp47_ = NULL;
	_tmp46_ = NULL;
	url = (_tmp47_ = (_tmp46_ = url, (_tmp46_ == NULL) ? NULL : g_strdup (_tmp46_)), url = (g_free (url), NULL), _tmp47_);
	net_set_redirection_status (self, status);
	result = NULL;
	internal_ip = (g_free (internal_ip), NULL);
	external_ip = (g_free (external_ip), NULL);
	url = (g_free (url), NULL);
	txtout = (g_free (txtout), NULL);
	txterr = (g_free (txterr), NULL);
	return result;
}


static gboolean _lambda3_ (Net* self) {
	gboolean result;
	net_set_redirection_status (self, NET_REDIRECTION_STATUS_NONE);
	result = FALSE;
	return result;
}


static gboolean __lambda3__gsource_func (gpointer self) {
	return _lambda3_ (self);
}


void net_forward_upnp_stop (Net* self) {
	GError * _inner_error_;
	char* txtout;
	char* txterr;
	gint _result_;
	gint status;
	gboolean _tmp0_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	txtout = NULL;
	txterr = NULL;
	_result_ = 0;
	status = 0;
	status = net_get_redirection_status (self);
	_tmp0_ = FALSE;
	if (status == NET_REDIRECTION_STATUS_DONE) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = status == NET_REDIRECTION_STATUS_PENDING;
	}
	if (_tmp0_) {
		char* _tmp2_;
		char* _tmp1_;
		gboolean _tmp3_;
		_tmp2_ = NULL;
		_tmp1_ = NULL;
		if ((_tmp3_ = _vala_strcmp0 (_tmp1_ = net_get_internal_ip (self), _tmp2_ = net_get_external_ip (self)) != 0, _tmp2_ = (g_free (_tmp2_), NULL), _tmp1_ = (g_free (_tmp1_), NULL), _tmp3_)) {
			char* _tmp11_;
			gboolean _tmp10_;
			char* _tmp9_;
			char* _tmp8_;
			gboolean _tmp7_;
			char* _tmp6_;
			char* _tmp5_;
			char* _tmp4_;
			_tmp11_ = NULL;
			_tmp9_ = NULL;
			_tmp8_ = NULL;
			_tmp6_ = NULL;
			_tmp5_ = NULL;
			_tmp4_ = NULL;
			_tmp10_ = (_tmp7_ = g_spawn_command_line_sync (_tmp5_ = g_strconcat (BINDIR, _tmp4_ = g_strdup_printf ("/fwupnp -d %d", net_get_port (self)), NULL), &_tmp6_, &_tmp9_, &_result_, &_inner_error_), txtout = (_tmp8_ = _tmp6_, txtout = (g_free (txtout), NULL), _tmp8_), _tmp7_);
			txterr = (_tmp11_ = _tmp9_, txterr = (g_free (txterr), NULL), _tmp11_);
			_tmp10_;
			if (_inner_error_ != NULL) {
				txtout = (g_free (txtout), NULL);
				txterr = (g_free (txterr), NULL);
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
			_tmp5_ = (g_free (_tmp5_), NULL);
			_tmp4_ = (g_free (_tmp4_), NULL);
			if (_result_ == 0) {
				net_log (self, _ ("Redirection removed"));
			} else {
				net_log (self, _ ("Unable to remove redirection"));
			}
		}
	}
	g_idle_add (__lambda3__gsource_func, self);
	txtout = (g_free (txtout), NULL);
	txterr = (g_free (txterr), NULL);
}


static gboolean _lambda7_ (Net* self) {
	gboolean result;
	net_set_redirection_status (self, NET_REDIRECTION_STATUS_PENDING);
	result = FALSE;
	return result;
}


static gboolean __lambda7__gsource_func (gpointer self) {
	return _lambda7_ (self);
}


static gboolean _lambda8_ (Net* self) {
	gboolean result;
	net_set_redirection_status (self, NET_REDIRECTION_STATUS_ERROR);
	result = FALSE;
	return result;
}


static gboolean __lambda8__gsource_func (gpointer self) {
	return _lambda8_ (self);
}


static void* net_forward_ssh_start (Net* self) {
	void* result;
	GError * _inner_error_;
	char* internal_ip;
	char* external_ip;
	char* url;
	char* txtout;
	char* txterr;
	gboolean bresult;
	gint _result_;
	gint status;
	char* _tmp5_;
	gboolean _tmp4_;
	char* _tmp3_;
	char* _tmp2_;
	gboolean _tmp1_;
	char* _tmp0_;
	char* _tmp11_;
	const char* _tmp10_;
	char* _tmp29_;
	char* _tmp31_;
	const char* _tmp30_;
	char* _tmp33_;
	const char* _tmp32_;
	char* _tmp35_;
	const char* _tmp34_;
	g_return_val_if_fail (self != NULL, NULL);
	_inner_error_ = NULL;
	internal_ip = NULL;
	external_ip = NULL;
	url = NULL;
	txtout = NULL;
	txterr = NULL;
	bresult = FALSE;
	_result_ = 0;
	status = 0;
	status = NET_REDIRECTION_STATUS_PENDING;
	g_idle_add (__lambda7__gsource_func, self);
	_tmp5_ = NULL;
	_tmp3_ = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	_tmp4_ = (_tmp1_ = g_spawn_command_line_sync (BINDIR "/fwlocalip", &_tmp0_, &_tmp3_, &_result_, &_inner_error_), txtout = (_tmp2_ = _tmp0_, txtout = (g_free (txtout), NULL), _tmp2_), _tmp1_);
	txterr = (_tmp5_ = _tmp3_, txterr = (g_free (txterr), NULL), _tmp5_);
	_tmp4_;
	if (_inner_error_ != NULL) {
		internal_ip = (g_free (internal_ip), NULL);
		external_ip = (g_free (external_ip), NULL);
		url = (g_free (url), NULL);
		txtout = (g_free (txtout), NULL);
		txterr = (g_free (txterr), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	if (_result_ == 0) {
		char* _tmp7_;
		const char* _tmp6_;
		char* _tmp8_;
		_tmp7_ = NULL;
		_tmp6_ = NULL;
		internal_ip = (_tmp7_ = (_tmp6_ = g_strchomp (txtout), (_tmp6_ == NULL) ? NULL : g_strdup (_tmp6_)), internal_ip = (g_free (internal_ip), NULL), _tmp7_);
		_tmp8_ = NULL;
		net_log (self, _tmp8_ = g_strdup_printf (_ ("Found internal IP: %s"), internal_ip));
		_tmp8_ = (g_free (_tmp8_), NULL);
	} else {
		char* _tmp9_;
		net_log (self, _ ("Local IP not found"));
		_tmp9_ = NULL;
		internal_ip = (_tmp9_ = g_strdup ("127.0.0.1"), internal_ip = (g_free (internal_ip), NULL), _tmp9_);
		g_idle_add (__lambda8__gsource_func, self);
		result = NULL;
		internal_ip = (g_free (internal_ip), NULL);
		external_ip = (g_free (external_ip), NULL);
		url = (g_free (url), NULL);
		txtout = (g_free (txtout), NULL);
		txterr = (g_free (txterr), NULL);
		return result;
	}
	_tmp11_ = NULL;
	_tmp10_ = NULL;
	external_ip = (_tmp11_ = (_tmp10_ = internal_ip, (_tmp10_ == NULL) ? NULL : g_strdup (_tmp10_)), external_ip = (g_free (external_ip), NULL), _tmp11_);
	/* Don't redirect if there's no valid internal IP
	 or remote host*/
	if (strcmp (internal_ip, "127.0.0.1") != 0) {
		char* sshcmd;
		gint sshcmd_argv_size;
		gint sshcmd_argv_length1;
		char** sshcmd_argv;
		char* _tmp12_;
		char* _tmp15_;
		char* _tmp14_;
		char* _tmp13_;
		char* _tmp16_;
		char** _tmp18_;
		char** _tmp17_;
		char* _tmp19_;
		char* _tmp20_;
		char* _tmp21_;
		char* _tmp22_;
		sshcmd = g_strdup (BINDIR "/fwssh -r ");
		sshcmd_argv = (sshcmd_argv_length1 = 0, NULL);
		_tmp12_ = NULL;
		external_ip = (_tmp12_ = net_get_ssh_host (self), external_ip = (g_free (external_ip), NULL), _tmp12_);
		_tmp15_ = NULL;
		_tmp14_ = NULL;
		_tmp13_ = NULL;
		sshcmd = (_tmp15_ = g_strconcat (sshcmd, _tmp14_ = g_strdup_printf ("%s %d %s %d", _tmp13_ = net_get_ssh_host (self), net_get_port (self), internal_ip, net_get_port (self)), NULL), sshcmd = (g_free (sshcmd), NULL), _tmp15_);
		_tmp14_ = (g_free (_tmp14_), NULL);
		_tmp13_ = (g_free (_tmp13_), NULL);
		_tmp16_ = NULL;
		net_log (self, _tmp16_ = g_strdup_printf (_ ("Using external IP: %s"), external_ip));
		_tmp16_ = (g_free (_tmp16_), NULL);
		net_log (self, _ ("Creating SSH tunnel"));
		_tmp18_ = NULL;
		_tmp17_ = NULL;
		sshcmd_argv = (_tmp18_ = _tmp17_ = g_strsplit_set (sshcmd, " ", 0), sshcmd_argv = (_vala_array_free (sshcmd_argv, sshcmd_argv_length1, (GDestroyNotify) g_free), NULL), sshcmd_argv_length1 = _vala_array_length (_tmp17_), sshcmd_argv_size = sshcmd_argv_length1, _tmp18_);
		_tmp19_ = NULL;
		g_setenv ("SSH_USER", _tmp19_ = net_get_ssh_user (self), TRUE);
		_tmp19_ = (g_free (_tmp19_), NULL);
		_tmp20_ = NULL;
		g_setenv ("SSH_PASSWORD", _tmp20_ = net_get_ssh_password (self), TRUE);
		_tmp20_ = (g_free (_tmp20_), NULL);
		_tmp21_ = NULL;
		g_setenv ("SSH_PORT", _tmp21_ = net_get_ssh_port (self), TRUE);
		_tmp21_ = (g_free (_tmp21_), NULL);
		_tmp22_ = NULL;
		g_setenv ("DISPLAY", _tmp22_ = net_get_display (self), TRUE);
		_tmp22_ = (g_free (_tmp22_), NULL);
		{
			gboolean _tmp23_;
			gboolean _tmp24_;
			_tmp23_ = g_spawn_sync (NULL, sshcmd_argv, NULL, G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL, NULL, NULL, NULL, NULL, &_result_, &_inner_error_);
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == G_SPAWN_ERROR) {
					goto __catch8_g_spawn_error;
				}
				goto __finally8;
			}
			bresult = _tmp23_;
			_tmp24_ = FALSE;
			if (bresult) {
				_tmp24_ = _result_ == 0;
			} else {
				_tmp24_ = FALSE;
			}
			if (_tmp24_) {
				net_log (self, _ ("Redirection performed"));
				status = NET_REDIRECTION_STATUS_DONE;
			} else {
				char* _tmp26_;
				const char* _tmp25_;
				net_log (self, _ ("Error creating SSH tunnel. Check config parameters and password."));
				_tmp26_ = NULL;
				_tmp25_ = NULL;
				external_ip = (_tmp26_ = (_tmp25_ = internal_ip, (_tmp25_ == NULL) ? NULL : g_strdup (_tmp25_)), external_ip = (g_free (external_ip), NULL), _tmp26_);
				status = NET_REDIRECTION_STATUS_ERROR;
			}
		}
		goto __finally8;
		__catch8_g_spawn_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
				char* _tmp28_;
				const char* _tmp27_;
				net_log (self, _ ("Error spawning SSH redirector process"));
				_tmp28_ = NULL;
				_tmp27_ = NULL;
				external_ip = (_tmp28_ = (_tmp27_ = internal_ip, (_tmp27_ == NULL) ? NULL : g_strdup (_tmp27_)), external_ip = (g_free (external_ip), NULL), _tmp28_);
				status = NET_REDIRECTION_STATUS_ERROR;
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally8:
		if (_inner_error_ != NULL) {
			sshcmd = (g_free (sshcmd), NULL);
			sshcmd_argv = (_vala_array_free (sshcmd_argv, sshcmd_argv_length1, (GDestroyNotify) g_free), NULL);
			internal_ip = (g_free (internal_ip), NULL);
			external_ip = (g_free (external_ip), NULL);
			url = (g_free (url), NULL);
			txtout = (g_free (txtout), NULL);
			txterr = (g_free (txterr), NULL);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return NULL;
		}
		sshcmd = (g_free (sshcmd), NULL);
		sshcmd_argv = (_vala_array_free (sshcmd_argv, sshcmd_argv_length1, (GDestroyNotify) g_free), NULL);
	}
	_tmp29_ = NULL;
	url = (_tmp29_ = g_strdup_printf ("http://%s:%d", external_ip, net_get_port (self)), url = (g_free (url), NULL), _tmp29_);
	_tmp31_ = NULL;
	_tmp30_ = NULL;
	internal_ip = (_tmp31_ = (_tmp30_ = internal_ip, (_tmp30_ == NULL) ? NULL : g_strdup (_tmp30_)), internal_ip = (g_free (internal_ip), NULL), _tmp31_);
	_tmp33_ = NULL;
	_tmp32_ = NULL;
	external_ip = (_tmp33_ = (_tmp32_ = external_ip, (_tmp32_ == NULL) ? NULL : g_strdup (_tmp32_)), external_ip = (g_free (external_ip), NULL), _tmp33_);
	_tmp35_ = NULL;
	_tmp34_ = NULL;
	url = (_tmp35_ = (_tmp34_ = url, (_tmp34_ == NULL) ? NULL : g_strdup (_tmp34_)), url = (g_free (url), NULL), _tmp35_);
	net_set_redirection_status (self, status);
	result = NULL;
	internal_ip = (g_free (internal_ip), NULL);
	external_ip = (g_free (external_ip), NULL);
	url = (g_free (url), NULL);
	txtout = (g_free (txtout), NULL);
	txterr = (g_free (txterr), NULL);
	return result;
}


static gboolean _lambda4_ (Net* self) {
	gboolean result;
	net_set_redirection_status (self, NET_REDIRECTION_STATUS_NONE);
	result = FALSE;
	return result;
}


static gboolean __lambda4__gsource_func (gpointer self) {
	return _lambda4_ (self);
}


void net_forward_ssh_stop (Net* self) {
	GError * _inner_error_;
	char* txtout;
	char* txterr;
	gint _result_;
	gint status;
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	txtout = NULL;
	txterr = NULL;
	_result_ = 0;
	status = 0;
	status = net_get_redirection_status (self);
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	if (status == NET_REDIRECTION_STATUS_DONE) {
		_tmp1_ = TRUE;
	} else {
		_tmp1_ = status == NET_REDIRECTION_STATUS_PENDING;
	}
	if (_tmp1_) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = status == NET_REDIRECTION_STATUS_ERROR;
	}
	if (_tmp0_) {
		char* _tmp3_;
		char* _tmp2_;
		gboolean _tmp4_;
		_tmp3_ = NULL;
		_tmp2_ = NULL;
		if ((_tmp4_ = _vala_strcmp0 (_tmp2_ = net_get_internal_ip (self), _tmp3_ = net_get_external_ip (self)) != 0, _tmp3_ = (g_free (_tmp3_), NULL), _tmp2_ = (g_free (_tmp2_), NULL), _tmp4_)) {
			char* _tmp12_;
			gboolean _tmp11_;
			char* _tmp10_;
			char* _tmp9_;
			gboolean _tmp8_;
			char* _tmp7_;
			char* _tmp6_;
			char* _tmp5_;
			_tmp12_ = NULL;
			_tmp10_ = NULL;
			_tmp9_ = NULL;
			_tmp7_ = NULL;
			_tmp6_ = NULL;
			_tmp5_ = NULL;
			_tmp11_ = (_tmp8_ = g_spawn_command_line_sync (_tmp6_ = g_strconcat (BINDIR, _tmp5_ = g_strdup_printf ("/fwssh -d %d", net_get_port (self)), NULL), &_tmp7_, &_tmp10_, &_result_, &_inner_error_), txtout = (_tmp9_ = _tmp7_, txtout = (g_free (txtout), NULL), _tmp9_), _tmp8_);
			txterr = (_tmp12_ = _tmp10_, txterr = (g_free (txterr), NULL), _tmp12_);
			_tmp11_;
			if (_inner_error_ != NULL) {
				txtout = (g_free (txtout), NULL);
				txterr = (g_free (txterr), NULL);
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
			_tmp6_ = (g_free (_tmp6_), NULL);
			_tmp5_ = (g_free (_tmp5_), NULL);
			if (_result_ == 0) {
				net_log (self, _ ("Redirection removed"));
			} else {
				net_log (self, _ ("Unable to remove redirection"));
			}
		}
	}
	g_idle_add (__lambda4__gsource_func, self);
	txtout = (g_free (txtout), NULL);
	txterr = (g_free (txterr), NULL);
}


static char* net_get_external_ip (Net* self) {
	char* result;
	char* r;
	g_return_val_if_fail (self != NULL, NULL);
	r = NULL;
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		char* _tmp1_;
		const char* _tmp0_;
		_tmp1_ = NULL;
		_tmp0_ = NULL;
		r = (_tmp1_ = (_tmp0_ = self->priv->_external_ip, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), r = (g_free (r), NULL), _tmp1_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	result = r;
	return result;
}


static void net_set_external_ip (Net* self, const char* value) {
	g_return_if_fail (self != NULL);
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		char* _tmp3_;
		const char* _tmp2_;
		_tmp3_ = NULL;
		_tmp2_ = NULL;
		self->priv->_external_ip = (_tmp3_ = (_tmp2_ = value, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), self->priv->_external_ip = (g_free (self->priv->_external_ip), NULL), _tmp3_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
}


static char* net_get_internal_ip (Net* self) {
	char* result;
	char* r;
	g_return_val_if_fail (self != NULL, NULL);
	r = NULL;
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		char* _tmp1_;
		const char* _tmp0_;
		_tmp1_ = NULL;
		_tmp0_ = NULL;
		r = (_tmp1_ = (_tmp0_ = self->priv->_internal_ip, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), r = (g_free (r), NULL), _tmp1_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	result = r;
	return result;
}


static void net_set_internal_ip (Net* self, const char* value) {
	g_return_if_fail (self != NULL);
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		char* _tmp3_;
		const char* _tmp2_;
		_tmp3_ = NULL;
		_tmp2_ = NULL;
		self->priv->_internal_ip = (_tmp3_ = (_tmp2_ = value, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), self->priv->_internal_ip = (g_free (self->priv->_internal_ip), NULL), _tmp3_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
}


gint net_get_port (Net* self) {
	gint result;
	gint r;
	g_return_val_if_fail (self != NULL, 0);
	r = 0;
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		r = self->priv->_port;
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	result = r;
	return result;
}


void net_set_port (Net* self, gint value) {
	g_return_if_fail (self != NULL);
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		self->priv->_port = value;
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	g_object_notify ((GObject *) self, "port");
}


char* net_get_url (Net* self) {
	char* result;
	char* r;
	g_return_val_if_fail (self != NULL, NULL);
	r = NULL;
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		const char* _tmp0_;
		char* _tmp2_;
		const char* _tmp1_;
		_tmp0_ = NULL;
		if (self->priv->_url == NULL) {
			_tmp0_ = "";
		} else {
			_tmp0_ = self->priv->_url;
		}
		_tmp2_ = NULL;
		_tmp1_ = NULL;
		r = (_tmp2_ = (_tmp1_ = _tmp0_, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), r = (g_free (r), NULL), _tmp2_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	result = r;
	return result;
}


static void net_set_url (Net* self, const char* value) {
	g_return_if_fail (self != NULL);
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		char* _tmp4_;
		const char* _tmp3_;
		_tmp4_ = NULL;
		_tmp3_ = NULL;
		self->priv->_url = (_tmp4_ = (_tmp3_ = value, (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_)), self->priv->_url = (g_free (self->priv->_url), NULL), _tmp4_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	g_object_notify ((GObject *) self, "url");
}


gint net_get_redirection_type (Net* self) {
	gint result;
	gint r;
	g_return_val_if_fail (self != NULL, 0);
	r = 0;
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		r = self->priv->_redirection_type;
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	result = r;
	return result;
}


static gboolean _net_forward_reload_gsource_func (gpointer self) {
	return net_forward_reload (self);
}


void net_set_redirection_type (Net* self, gint value) {
	g_return_if_fail (self != NULL);
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		/* Don't allow type changing in the middle of a change process*/
		if (self->priv->_redirection_status == NET_REDIRECTION_STATUS_PENDING) {
			return;
		}
		self->priv->_previous_redirection_type = self->priv->_redirection_type;
		self->priv->_redirection_type = value;
		g_idle_add (_net_forward_reload_gsource_func, self);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	g_object_notify ((GObject *) self, "redirection-type");
}


gint net_get_redirection_status (Net* self) {
	gint result;
	gint r;
	g_return_val_if_fail (self != NULL, 0);
	r = 0;
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		r = self->priv->_redirection_status;
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	result = r;
	return result;
}


void net_set_redirection_status (Net* self, gint value) {
	g_return_if_fail (self != NULL);
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		self->priv->_redirection_status = value;
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	g_object_notify ((GObject *) self, "redirection-status");
}


char* net_get_ssh_host (Net* self) {
	char* result;
	char* r;
	g_return_val_if_fail (self != NULL, NULL);
	r = NULL;
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		char* _tmp1_;
		const char* _tmp0_;
		_tmp1_ = NULL;
		_tmp0_ = NULL;
		r = (_tmp1_ = (_tmp0_ = self->priv->_ssh_host, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), r = (g_free (r), NULL), _tmp1_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	result = r;
	return result;
}


void net_set_ssh_host (Net* self, const char* value) {
	g_return_if_fail (self != NULL);
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		char* _tmp3_;
		const char* _tmp2_;
		_tmp3_ = NULL;
		_tmp2_ = NULL;
		self->priv->_ssh_host = (_tmp3_ = (_tmp2_ = value, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), self->priv->_ssh_host = (g_free (self->priv->_ssh_host), NULL), _tmp3_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	g_object_notify ((GObject *) self, "ssh-host");
}


char* net_get_ssh_port (Net* self) {
	char* result;
	char* r;
	g_return_val_if_fail (self != NULL, NULL);
	r = NULL;
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		char* _tmp1_;
		const char* _tmp0_;
		_tmp1_ = NULL;
		_tmp0_ = NULL;
		r = (_tmp1_ = (_tmp0_ = self->priv->_ssh_port, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), r = (g_free (r), NULL), _tmp1_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	result = r;
	return result;
}


void net_set_ssh_port (Net* self, const char* value) {
	g_return_if_fail (self != NULL);
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		char* _tmp3_;
		const char* _tmp2_;
		_tmp3_ = NULL;
		_tmp2_ = NULL;
		self->priv->_ssh_port = (_tmp3_ = (_tmp2_ = value, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), self->priv->_ssh_port = (g_free (self->priv->_ssh_port), NULL), _tmp3_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	g_object_notify ((GObject *) self, "ssh-port");
}


char* net_get_ssh_user (Net* self) {
	char* result;
	char* r;
	g_return_val_if_fail (self != NULL, NULL);
	r = NULL;
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		char* _tmp1_;
		const char* _tmp0_;
		_tmp1_ = NULL;
		_tmp0_ = NULL;
		r = (_tmp1_ = (_tmp0_ = self->priv->_ssh_user, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), r = (g_free (r), NULL), _tmp1_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	result = r;
	return result;
}


void net_set_ssh_user (Net* self, const char* value) {
	g_return_if_fail (self != NULL);
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		char* _tmp3_;
		const char* _tmp2_;
		_tmp3_ = NULL;
		_tmp2_ = NULL;
		self->priv->_ssh_user = (_tmp3_ = (_tmp2_ = value, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), self->priv->_ssh_user = (g_free (self->priv->_ssh_user), NULL), _tmp3_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	g_object_notify ((GObject *) self, "ssh-user");
}


char* net_get_ssh_password (Net* self) {
	char* result;
	char* r;
	g_return_val_if_fail (self != NULL, NULL);
	r = NULL;
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		char* _tmp1_;
		const char* _tmp0_;
		_tmp1_ = NULL;
		_tmp0_ = NULL;
		r = (_tmp1_ = (_tmp0_ = self->priv->_ssh_password, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), r = (g_free (r), NULL), _tmp1_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	result = r;
	return result;
}


void net_set_ssh_password (Net* self, const char* value) {
	g_return_if_fail (self != NULL);
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		char* _tmp3_;
		const char* _tmp2_;
		_tmp3_ = NULL;
		_tmp2_ = NULL;
		self->priv->_ssh_password = (_tmp3_ = (_tmp2_ = value, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), self->priv->_ssh_password = (g_free (self->priv->_ssh_password), NULL), _tmp3_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	g_object_notify ((GObject *) self, "ssh-password");
}


char* net_get_display (Net* self) {
	char* result;
	char* r;
	g_return_val_if_fail (self != NULL, NULL);
	r = NULL;
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		char* _tmp1_;
		const char* _tmp0_;
		_tmp1_ = NULL;
		_tmp0_ = NULL;
		r = (_tmp1_ = (_tmp0_ = self->priv->_display, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), r = (g_free (r), NULL), _tmp1_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	result = r;
	return result;
}


void net_set_display (Net* self, const char* value) {
	g_return_if_fail (self != NULL);
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		char* _tmp3_;
		const char* _tmp2_;
		_tmp3_ = NULL;
		_tmp2_ = NULL;
		self->priv->_display = (_tmp3_ = (_tmp2_ = value, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), self->priv->_display = (g_free (self->priv->_display), NULL), _tmp3_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	g_object_notify ((GObject *) self, "display");
}


static Log* net_get_logger (Net* self) {
	Log* result;
	Log* r;
	g_return_val_if_fail (self != NULL, NULL);
	r = NULL;
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		Log* _tmp1_;
		Log* _tmp0_;
		_tmp1_ = NULL;
		_tmp0_ = NULL;
		r = (_tmp1_ = (_tmp0_ = self->priv->_logger, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (r == NULL) ? NULL : (r = (g_object_unref (r), NULL)), _tmp1_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	result = r;
	return result;
}


void net_set_logger (Net* self, Log* value) {
	g_return_if_fail (self != NULL);
	g_static_rec_mutex_lock (&self->priv->__lock_worker);
	{
		Log* _tmp3_;
		Log* _tmp2_;
		_tmp3_ = NULL;
		_tmp2_ = NULL;
		self->priv->_logger = (_tmp3_ = (_tmp2_ = value, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (self->priv->_logger == NULL) ? NULL : (self->priv->_logger = (g_object_unref (self->priv->_logger), NULL)), _tmp3_);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_worker);
	g_object_notify ((GObject *) self, "logger");
}


static void net_class_init (NetClass * klass) {
	net_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (NetPrivate));
	G_OBJECT_CLASS (klass)->get_property = net_get_property;
	G_OBJECT_CLASS (klass)->set_property = net_set_property;
	G_OBJECT_CLASS (klass)->finalize = net_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), NET_PORT, g_param_spec_int ("port", "port", "port", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), NET_URL, g_param_spec_string ("url", "url", "url", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), NET_REDIRECTION_TYPE, g_param_spec_int ("redirection-type", "redirection-type", "redirection-type", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), NET_REDIRECTION_STATUS, g_param_spec_int ("redirection-status", "redirection-status", "redirection-status", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), NET_SSH_HOST, g_param_spec_string ("ssh-host", "ssh-host", "ssh-host", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), NET_SSH_PORT, g_param_spec_string ("ssh-port", "ssh-port", "ssh-port", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), NET_SSH_USER, g_param_spec_string ("ssh-user", "ssh-user", "ssh-user", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), NET_SSH_PASSWORD, g_param_spec_string ("ssh-password", "ssh-password", "ssh-password", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), NET_DISPLAY, g_param_spec_string ("display", "display", "display", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), NET_LOGGER, g_param_spec_object ("logger", "logger", "logger", TYPE_LOG, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE));
}


static void net_instance_init (Net * self) {
	self->priv = NET_GET_PRIVATE (self);
	g_static_rec_mutex_init (&self->priv->__lock_worker);
	self->priv->worker = NULL;
	self->priv->_url = NULL;
	self->priv->_display = NULL;
}


static void net_finalize (GObject* obj) {
	Net * self;
	self = NET (obj);
	g_static_rec_mutex_free (&self->priv->__lock_worker);
	self->priv->_external_ip = (g_free (self->priv->_external_ip), NULL);
	self->priv->_internal_ip = (g_free (self->priv->_internal_ip), NULL);
	self->priv->_url = (g_free (self->priv->_url), NULL);
	self->priv->_ssh_host = (g_free (self->priv->_ssh_host), NULL);
	self->priv->_ssh_port = (g_free (self->priv->_ssh_port), NULL);
	self->priv->_ssh_user = (g_free (self->priv->_ssh_user), NULL);
	self->priv->_ssh_password = (g_free (self->priv->_ssh_password), NULL);
	self->priv->_display = (g_free (self->priv->_display), NULL);
	(self->priv->_logger == NULL) ? NULL : (self->priv->_logger = (g_object_unref (self->priv->_logger), NULL));
	G_OBJECT_CLASS (net_parent_class)->finalize (obj);
}


GType net_get_type (void) {
	static GType net_type_id = 0;
	if (net_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (NetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) net_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Net), 0, (GInstanceInitFunc) net_instance_init, NULL };
		net_type_id = g_type_register_static (G_TYPE_OBJECT, "Net", &g_define_type_info, 0);
	}
	return net_type_id;
}


static void net_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Net * self;
	gpointer boxed;
	self = NET (object);
	switch (property_id) {
		case NET_PORT:
		g_value_set_int (value, net_get_port (self));
		break;
		case NET_URL:
		g_value_set_string (value, net_get_url (self));
		break;
		case NET_REDIRECTION_TYPE:
		g_value_set_int (value, net_get_redirection_type (self));
		break;
		case NET_REDIRECTION_STATUS:
		g_value_set_int (value, net_get_redirection_status (self));
		break;
		case NET_SSH_HOST:
		g_value_set_string (value, net_get_ssh_host (self));
		break;
		case NET_SSH_PORT:
		g_value_set_string (value, net_get_ssh_port (self));
		break;
		case NET_SSH_USER:
		g_value_set_string (value, net_get_ssh_user (self));
		break;
		case NET_SSH_PASSWORD:
		g_value_set_string (value, net_get_ssh_password (self));
		break;
		case NET_DISPLAY:
		g_value_set_string (value, net_get_display (self));
		break;
		case NET_LOGGER:
		g_value_set_object (value, net_get_logger (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void net_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Net * self;
	self = NET (object);
	switch (property_id) {
		case NET_PORT:
		net_set_port (self, g_value_get_int (value));
		break;
		case NET_URL:
		net_set_url (self, g_value_get_string (value));
		break;
		case NET_REDIRECTION_TYPE:
		net_set_redirection_type (self, g_value_get_int (value));
		break;
		case NET_REDIRECTION_STATUS:
		net_set_redirection_status (self, g_value_get_int (value));
		break;
		case NET_SSH_HOST:
		net_set_ssh_host (self, g_value_get_string (value));
		break;
		case NET_SSH_PORT:
		net_set_ssh_port (self, g_value_get_string (value));
		break;
		case NET_SSH_USER:
		net_set_ssh_user (self, g_value_get_string (value));
		break;
		case NET_SSH_PASSWORD:
		net_set_ssh_password (self, g_value_get_string (value));
		break;
		case NET_DISPLAY:
		net_set_display (self, g_value_get_string (value));
		break;
		case NET_LOGGER:
		net_set_logger (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




