/* Meiga - Lightweight and easy to use web file server for your desktop
 *
 * Copyright (C) 2009 Igalia, S.L.
 *
 * Authors:
 *
 * Igalia, S.L. <info@igalia.com>
 * Enrique Ocaña González <eocanha@igalia.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <glib/gi18n-lib.h>
#include <gio/gio.h>
#include <stdio.h>


#define TYPE_RSS_NODE (rss_node_get_type ())
#define RSS_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_RSS_NODE, RssNode))
#define RSS_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_RSS_NODE, RssNodeClass))
#define IS_RSS_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_RSS_NODE))
#define IS_RSS_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_RSS_NODE))
#define RSS_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_RSS_NODE, RssNodeClass))

typedef struct _RssNode RssNode;
typedef struct _RssNodeClass RssNodeClass;
typedef struct _RssNodePrivate RssNodePrivate;

#define TYPE_RSS_FEED (rss_feed_get_type ())
#define RSS_FEED(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_RSS_FEED, RssFeed))
#define RSS_FEED_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_RSS_FEED, RssFeedClass))
#define IS_RSS_FEED(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_RSS_FEED))
#define IS_RSS_FEED_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_RSS_FEED))
#define RSS_FEED_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_RSS_FEED, RssFeedClass))

typedef struct _RssFeed RssFeed;
typedef struct _RssFeedClass RssFeedClass;
typedef struct _RssFeedPrivate RssFeedPrivate;

struct _RssNode {
	GObject parent_instance;
	RssNodePrivate * priv;
};

struct _RssNodeClass {
	GObjectClass parent_class;
};

struct _RssNodePrivate {
	char* _title;
	char* _link;
	struct tm _time;
	char* _description;
	char* _content;
};

struct _RssFeed {
	GObject parent_instance;
	RssFeedPrivate * priv;
	GList* nodes;
};

struct _RssFeedClass {
	GObjectClass parent_class;
};

struct _RssFeedPrivate {
	char* _title;
	char* _link;
	char* _description;
	struct tm _time;
	char* _generator;
	char* _language;
};


static gpointer rss_node_parent_class = NULL;
static gpointer rss_feed_parent_class = NULL;

GType rss_node_get_type (void);
#define RSS_NODE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_RSS_NODE, RssNodePrivate))
enum  {
	RSS_NODE_DUMMY_PROPERTY,
	RSS_NODE_TITLE,
	RSS_NODE_LINK,
	RSS_NODE_DESCRIPTION,
	RSS_NODE_CONTENT
};
void rss_node_set_time (RssNode* self, struct tm* value);
RssNode* rss_node_new (void);
RssNode* rss_node_construct (GType object_type);
const char* rss_node_get_title (RssNode* self);
const char* rss_node_get_link (RssNode* self);
void rss_node_get_time (RssNode* self, struct tm* value);
const char* rss_node_get_description (RssNode* self);
const char* rss_node_get_content (RssNode* self);
char* rss_node_to_string (RssNode* self);
void rss_node_set_title (RssNode* self, const char* value);
void rss_node_set_link (RssNode* self, const char* value);
void rss_node_set_description (RssNode* self, const char* value);
void rss_node_set_content (RssNode* self, const char* value);
static void rss_node_finalize (GObject* obj);
static void rss_node_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void rss_node_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType rss_feed_get_type (void);
#define RSS_FEED_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_RSS_FEED, RssFeedPrivate))
enum  {
	RSS_FEED_DUMMY_PROPERTY,
	RSS_FEED_TITLE,
	RSS_FEED_LINK,
	RSS_FEED_DESCRIPTION,
	RSS_FEED_GENERATOR,
	RSS_FEED_LANGUAGE
};
static void _g_list_free_g_object_unref (GList* self);
void rss_feed_set_time (RssFeed* self, struct tm* value);
RssFeed* rss_feed_new (void);
RssFeed* rss_feed_construct (GType object_type);
const char* rss_feed_get_generator (RssFeed* self);
const char* rss_feed_get_title (RssFeed* self);
const char* rss_feed_get_link (RssFeed* self);
const char* rss_feed_get_description (RssFeed* self);
void rss_feed_get_time (RssFeed* self, struct tm* value);
const char* rss_feed_get_language (RssFeed* self);
char* rss_feed_to_string (RssFeed* self);
void rss_feed_set_title (RssFeed* self, const char* value);
void rss_feed_set_description (RssFeed* self, const char* value);
RssFeed* rss_feed_new_from_directory (const char* path, const char* base_url, const char* title, const char* description);
void rss_feed_set_link (RssFeed* self, const char* value);
void rss_feed_set_generator (RssFeed* self, const char* value);
void rss_feed_set_language (RssFeed* self, const char* value);
static void rss_feed_finalize (GObject* obj);
static void rss_feed_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void rss_feed_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static int _vala_strcmp0 (const char * str1, const char * str2);



RssNode* rss_node_construct (GType object_type) {
	RssNode * self;
	struct tm _tmp0_ = {0};
	struct tm _tmp1_ = {0};
	self = g_object_newv (object_type, 0, NULL);
	rss_node_set_time (self, (_tmp1_ = (memset (&_tmp0_, 0, sizeof (struct tm)), _tmp0_), &_tmp1_));
	return self;
}


RssNode* rss_node_new (void) {
	return rss_node_construct (TYPE_RSS_NODE);
}


static char* g_time_to_string (struct tm *self) {
	char* result;
	result = g_strdup_printf ("%04d-%02d-%02d %02d:%02d:%02d", (*self).tm_year + 1900, (*self).tm_mon + 1, (*self).tm_mday, (*self).tm_hour, (*self).tm_min, (*self).tm_sec);
	return result;
}


char* rss_node_to_string (RssNode* self) {
	char* result;
	char* _tmp17_;
	char* _tmp16_;
	char* _tmp15_;
	char* _tmp14_;
	char* _tmp13_;
	char* _tmp12_;
	char* _tmp11_;
	char* _tmp10_;
	char* _tmp9_;
	char* _tmp8_;
	char* _tmp7_;
	char* _tmp6_;
	char* _tmp5_;
	char* _tmp4_;
	char* _tmp3_;
	char* _tmp2_;
	char* _tmp1_;
	char* _tmp0_;
	char* _tmp18_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp17_ = NULL;
	_tmp16_ = NULL;
	_tmp15_ = NULL;
	_tmp14_ = NULL;
	_tmp13_ = NULL;
	_tmp12_ = NULL;
	_tmp11_ = NULL;
	_tmp10_ = NULL;
	_tmp9_ = NULL;
	_tmp8_ = NULL;
	_tmp7_ = NULL;
	_tmp6_ = NULL;
	_tmp5_ = NULL;
	_tmp4_ = NULL;
	_tmp3_ = NULL;
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	_tmp18_ = NULL;
	result = (_tmp18_ = g_strconcat (_tmp17_ = g_strconcat (_tmp16_ = g_strconcat (_tmp15_ = g_strconcat (_tmp14_ = g_strconcat (_tmp13_ = g_strconcat (_tmp12_ = g_strconcat (_tmp11_ = g_strconcat (_tmp10_ = g_strconcat (_tmp9_ = g_strconcat (_tmp8_ = g_strconcat (_tmp6_ = g_strconcat (_tmp5_ = g_strconcat (_tmp4_ = g_strconcat (_tmp3_ = g_strconcat (_tmp2_ = g_strconcat (_tmp1_ = g_strconcat (_tmp0_ = g_strconcat ("   <title>", self->priv->_title, NULL), "</title>\n", NULL), "   <link>", NULL), self->priv->_link, NULL), "</link>\n", NULL), "\n", NULL), "   <pubDate>", NULL), _tmp7_ = g_time_to_string (&self->priv->_time), NULL), "</pubDate>\n", NULL), "   <guid>", NULL), self->priv->_link, NULL), "</guid>\n", NULL), "   <description><![CDATA[", NULL), self->priv->_description, NULL), "]]></description>\n", NULL), "     <content:encoded><![CDATA[", NULL), self->priv->_content, NULL), "]]></content:encoded>\n", NULL), _tmp17_ = (g_free (_tmp17_), NULL), _tmp16_ = (g_free (_tmp16_), NULL), _tmp15_ = (g_free (_tmp15_), NULL), _tmp14_ = (g_free (_tmp14_), NULL), _tmp13_ = (g_free (_tmp13_), NULL), _tmp12_ = (g_free (_tmp12_), NULL), _tmp11_ = (g_free (_tmp11_), NULL), _tmp10_ = (g_free (_tmp10_), NULL), _tmp9_ = (g_free (_tmp9_), NULL), _tmp8_ = (g_free (_tmp8_), NULL), _tmp7_ = (g_free (_tmp7_), NULL), _tmp6_ = (g_free (_tmp6_), NULL), _tmp5_ = (g_free (_tmp5_), NULL), _tmp4_ = (g_free (_tmp4_), NULL), _tmp3_ = (g_free (_tmp3_), NULL), _tmp2_ = (g_free (_tmp2_), NULL), _tmp1_ = (g_free (_tmp1_), NULL), _tmp0_ = (g_free (_tmp0_), NULL), _tmp18_);
	return result;
}


const char* rss_node_get_title (RssNode* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_title;
	return result;
}


void rss_node_set_title (RssNode* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_title = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_title = (g_free (self->priv->_title), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "title");
}


const char* rss_node_get_link (RssNode* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_link;
	return result;
}


void rss_node_set_link (RssNode* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_link = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_link = (g_free (self->priv->_link), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "link");
}


void rss_node_get_time (RssNode* self, struct tm* value) {
	struct tm result;
	g_return_if_fail (self != NULL);
	result = self->priv->_time;
	*value = result;
	return;
}


void rss_node_set_time (RssNode* self, struct tm* value) {
	g_return_if_fail (self != NULL);
	self->priv->_time = (*value);
}


const char* rss_node_get_description (RssNode* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_description;
	return result;
}


void rss_node_set_description (RssNode* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_description = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_description = (g_free (self->priv->_description), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "description");
}


const char* rss_node_get_content (RssNode* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_content;
	return result;
}


void rss_node_set_content (RssNode* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_content = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_content = (g_free (self->priv->_content), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "content");
}


static void rss_node_class_init (RssNodeClass * klass) {
	rss_node_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (RssNodePrivate));
	G_OBJECT_CLASS (klass)->get_property = rss_node_get_property;
	G_OBJECT_CLASS (klass)->set_property = rss_node_set_property;
	G_OBJECT_CLASS (klass)->finalize = rss_node_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), RSS_NODE_TITLE, g_param_spec_string ("title", "title", "title", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), RSS_NODE_LINK, g_param_spec_string ("link", "link", "link", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), RSS_NODE_DESCRIPTION, g_param_spec_string ("description", "description", "description", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), RSS_NODE_CONTENT, g_param_spec_string ("content", "content", "content", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void rss_node_instance_init (RssNode * self) {
	const char* _tmp0_;
	const char* _tmp1_;
	const char* _tmp2_;
	self->priv = RSS_NODE_GET_PRIVATE (self);
	self->priv->_title = (_tmp0_ = _ ("No title"), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	_tmp0_ = NULL;
	self->priv->_link = g_strdup ("");
	self->priv->_description = (_tmp1_ = _ ("No description"), (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_));
	_tmp1_ = NULL;
	self->priv->_content = (_tmp2_ = _ ("No content"), (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_));
	_tmp2_ = NULL;
}


static void rss_node_finalize (GObject* obj) {
	RssNode * self;
	self = RSS_NODE (obj);
	self->priv->_title = (g_free (self->priv->_title), NULL);
	self->priv->_link = (g_free (self->priv->_link), NULL);
	self->priv->_description = (g_free (self->priv->_description), NULL);
	self->priv->_content = (g_free (self->priv->_content), NULL);
	G_OBJECT_CLASS (rss_node_parent_class)->finalize (obj);
}


GType rss_node_get_type (void) {
	static GType rss_node_type_id = 0;
	if (rss_node_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (RssNodeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) rss_node_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (RssNode), 0, (GInstanceInitFunc) rss_node_instance_init, NULL };
		rss_node_type_id = g_type_register_static (G_TYPE_OBJECT, "RssNode", &g_define_type_info, 0);
	}
	return rss_node_type_id;
}


static void rss_node_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	RssNode * self;
	gpointer boxed;
	self = RSS_NODE (object);
	switch (property_id) {
		case RSS_NODE_TITLE:
		g_value_set_string (value, rss_node_get_title (self));
		break;
		case RSS_NODE_LINK:
		g_value_set_string (value, rss_node_get_link (self));
		break;
		case RSS_NODE_DESCRIPTION:
		g_value_set_string (value, rss_node_get_description (self));
		break;
		case RSS_NODE_CONTENT:
		g_value_set_string (value, rss_node_get_content (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void rss_node_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	RssNode * self;
	self = RSS_NODE (object);
	switch (property_id) {
		case RSS_NODE_TITLE:
		rss_node_set_title (self, g_value_get_string (value));
		break;
		case RSS_NODE_LINK:
		rss_node_set_link (self, g_value_get_string (value));
		break;
		case RSS_NODE_DESCRIPTION:
		rss_node_set_description (self, g_value_get_string (value));
		break;
		case RSS_NODE_CONTENT:
		rss_node_set_content (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


RssFeed* rss_feed_construct (GType object_type) {
	RssFeed * self;
	struct tm _tmp0_ = {0};
	struct tm _tmp1_ = {0};
	GList* _tmp2_;
	self = g_object_newv (object_type, 0, NULL);
	rss_feed_set_time (self, (_tmp1_ = (memset (&_tmp0_, 0, sizeof (struct tm)), _tmp0_), &_tmp1_));
	_tmp2_ = NULL;
	self->nodes = (_tmp2_ = NULL, (self->nodes == NULL) ? NULL : (self->nodes = (_g_list_free_g_object_unref (self->nodes), NULL)), _tmp2_);
	return self;
}


RssFeed* rss_feed_new (void) {
	return rss_feed_construct (TYPE_RSS_FEED);
}


char* rss_feed_to_string (RssFeed* self) {
	char* result;
	char* _tmp27_;
	char* _tmp26_;
	char* _tmp25_;
	char* _tmp24_;
	char* _tmp23_;
	char* _tmp22_;
	char* _tmp21_;
	char* _tmp20_;
	char* _tmp19_;
	char* _tmp18_;
	char* _tmp17_;
	char* _tmp16_;
	char* _tmp15_;
	char* _tmp14_;
	char* _tmp13_;
	char* _tmp12_;
	char* _tmp11_;
	char* _tmp10_;
	char* _tmp9_;
	char* _tmp8_;
	char* _tmp7_;
	char* _tmp6_;
	char* _tmp5_;
	char* _tmp4_;
	char* _tmp3_;
	char* _tmp2_;
	char* _tmp1_;
	char* _tmp0_;
	char* _tmp28_;
	char* _result_;
	char* _tmp34_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp27_ = NULL;
	_tmp26_ = NULL;
	_tmp25_ = NULL;
	_tmp24_ = NULL;
	_tmp23_ = NULL;
	_tmp22_ = NULL;
	_tmp21_ = NULL;
	_tmp20_ = NULL;
	_tmp19_ = NULL;
	_tmp18_ = NULL;
	_tmp17_ = NULL;
	_tmp16_ = NULL;
	_tmp15_ = NULL;
	_tmp14_ = NULL;
	_tmp13_ = NULL;
	_tmp12_ = NULL;
	_tmp11_ = NULL;
	_tmp10_ = NULL;
	_tmp9_ = NULL;
	_tmp8_ = NULL;
	_tmp7_ = NULL;
	_tmp6_ = NULL;
	_tmp5_ = NULL;
	_tmp4_ = NULL;
	_tmp3_ = NULL;
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	_tmp28_ = NULL;
	_result_ = (_tmp28_ = g_strconcat (_tmp27_ = g_strconcat (_tmp26_ = g_strconcat (_tmp25_ = g_strconcat (_tmp24_ = g_strconcat (_tmp23_ = g_strconcat (_tmp22_ = g_strconcat (_tmp21_ = g_strconcat (_tmp20_ = g_strconcat (_tmp18_ = g_strconcat (_tmp17_ = g_strconcat (_tmp16_ = g_strconcat (_tmp15_ = g_strconcat (_tmp14_ = g_strconcat (_tmp13_ = g_strconcat (_tmp12_ = g_strconcat (_tmp11_ = g_strconcat (_tmp10_ = g_strconcat (_tmp9_ = g_strconcat (_tmp8_ = g_strconcat (_tmp7_ = g_strconcat (_tmp6_ = g_strconcat (_tmp5_ = g_strconcat (_tmp4_ = g_strconcat (_tmp3_ = g_strconcat (_tmp2_ = g_strconcat (_tmp1_ = g_strconcat (_tmp0_ = g_strconcat ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" "<!-- generator=\"", self->priv->_generator, NULL), "\" -->\n", NULL), "<rss version=\"2.0\" \n", NULL), "\txmlns:content=\"http://purl.org/rss/1.0/modules/content/\"\n", NULL), "\txmlns:wfw=\"http://wellformedweb.org/CommentAPI/\"\n", NULL), "\txmlns:dc=\"http://purl.org/dc/elements/1.1/\"\n", NULL), "\t>\n", NULL), "\n", NULL), "\t<channel>\n", NULL), "\t\t<title>", NULL), self->priv->_title, NULL), "</title>\n", NULL), "\t\t<link>", NULL), self->priv->_link, NULL), "</link>\n", NULL), "\t\t<description>", NULL), self->priv->_description, NULL), "</description>\n", NULL), "\t\t<pubDate>", NULL), _tmp19_ = g_time_to_string (&self->priv->_time), NULL), "</pubDate>\n", NULL), "\n", NULL), "\t\t<generator>", NULL), self->priv->_generator, NULL), "</generator>\n", NULL), "\t\t<language>", NULL), self->priv->_language, NULL), "</language>\n", NULL), _tmp27_ = (g_free (_tmp27_), NULL), _tmp26_ = (g_free (_tmp26_), NULL), _tmp25_ = (g_free (_tmp25_), NULL), _tmp24_ = (g_free (_tmp24_), NULL), _tmp23_ = (g_free (_tmp23_), NULL), _tmp22_ = (g_free (_tmp22_), NULL), _tmp21_ = (g_free (_tmp21_), NULL), _tmp20_ = (g_free (_tmp20_), NULL), _tmp19_ = (g_free (_tmp19_), NULL), _tmp18_ = (g_free (_tmp18_), NULL), _tmp17_ = (g_free (_tmp17_), NULL), _tmp16_ = (g_free (_tmp16_), NULL), _tmp15_ = (g_free (_tmp15_), NULL), _tmp14_ = (g_free (_tmp14_), NULL), _tmp13_ = (g_free (_tmp13_), NULL), _tmp12_ = (g_free (_tmp12_), NULL), _tmp11_ = (g_free (_tmp11_), NULL), _tmp10_ = (g_free (_tmp10_), NULL), _tmp9_ = (g_free (_tmp9_), NULL), _tmp8_ = (g_free (_tmp8_), NULL), _tmp7_ = (g_free (_tmp7_), NULL), _tmp6_ = (g_free (_tmp6_), NULL), _tmp5_ = (g_free (_tmp5_), NULL), _tmp4_ = (g_free (_tmp4_), NULL), _tmp3_ = (g_free (_tmp3_), NULL), _tmp2_ = (g_free (_tmp2_), NULL), _tmp1_ = (g_free (_tmp1_), NULL), _tmp0_ = (g_free (_tmp0_), NULL), _tmp28_);
	{
		GList* node_collection;
		GList* node_it;
		node_collection = self->nodes;
		for (node_it = node_collection; node_it != NULL; node_it = node_it->next) {
			RssNode* _tmp33_;
			RssNode* node;
			_tmp33_ = NULL;
			node = (_tmp33_ = (RssNode*) node_it->data, (_tmp33_ == NULL) ? NULL : g_object_ref (_tmp33_));
			{
				char* _tmp32_;
				char* _tmp31_;
				char* _tmp30_;
				char* _tmp29_;
				_tmp32_ = NULL;
				_tmp31_ = NULL;
				_tmp30_ = NULL;
				_tmp29_ = NULL;
				_result_ = (_tmp32_ = g_strconcat (_result_, _tmp31_ = g_strconcat (_tmp30_ = g_strconcat ("\t\t<item>\n" "\t\t", _tmp29_ = rss_node_to_string (node), NULL), "\t\t</item>\n", NULL), NULL), _result_ = (g_free (_result_), NULL), _tmp32_);
				_tmp31_ = (g_free (_tmp31_), NULL);
				_tmp30_ = (g_free (_tmp30_), NULL);
				_tmp29_ = (g_free (_tmp29_), NULL);
				(node == NULL) ? NULL : (node = (g_object_unref (node), NULL));
			}
		}
	}
	_tmp34_ = NULL;
	_result_ = (_tmp34_ = g_strconcat (_result_, "\t</channel>\n" "</rss>\n", NULL), _result_ = (g_free (_result_), NULL), _tmp34_);
	result = _result_;
	return result;
}


RssFeed* rss_feed_new_from_directory (const char* path, const char* base_url, const char* title, const char* description) {
	RssFeed* result;
	GError * _inner_error_;
	RssFeed* rss;
	g_return_val_if_fail (path != NULL, NULL);
	g_return_val_if_fail (base_url != NULL, NULL);
	g_return_val_if_fail (title != NULL, NULL);
	g_return_val_if_fail (description != NULL, NULL);
	_inner_error_ = NULL;
	rss = rss_feed_new ();
	if (title != NULL) {
		rss_feed_set_title (rss, title);
	}
	if (description != NULL) {
		rss_feed_set_description (rss, description);
	}
	{
		GFile* dir;
		GFileInfo* info;
		guint64 time;
		time_t _tmp1_ = {0};
		struct tm _tmp0_ = {0};
		struct tm _tmp2_ = {0};
		GFileEnumerator* files;
		dir = g_file_new_for_path (path);
		info = g_file_query_info (dir, G_FILE_ATTRIBUTE_TIME_MODIFIED, G_FILE_QUERY_INFO_NONE, NULL, &_inner_error_);
		if (_inner_error_ != NULL) {
			(dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL));
			goto __catch5_g_error;
			goto __finally5;
		}
		time = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
		rss_feed_set_time (rss, (_tmp2_ = (localtime_r ((_tmp1_ = (time_t) time, &_tmp1_), &_tmp0_), _tmp0_), &_tmp2_));
		files = g_file_enumerate_children (dir, "standard::*," G_FILE_ATTRIBUTE_TIME_MODIFIED, G_FILE_QUERY_INFO_NONE, NULL, &_inner_error_);
		if (_inner_error_ != NULL) {
			(dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL));
			(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
			goto __catch5_g_error;
			goto __finally5;
		}
		while (TRUE) {
			GFileInfo* _tmp3_;
			GFileInfo* _tmp4_;
			RssNode* node;
			const char* _tmp5_;
			char* name;
			char* _tmp7_;
			char* _tmp6_;
			time_t _tmp9_ = {0};
			struct tm _tmp8_ = {0};
			struct tm _tmp10_ = {0};
			GFileType type;
			const char* _tmp11_;
			char* mimetype;
			RssNode* _tmp39_;
			_tmp3_ = g_file_enumerator_next_file (files, NULL, &_inner_error_);
			if (_inner_error_ != NULL) {
				(dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL));
				(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
				(files == NULL) ? NULL : (files = (g_object_unref (files), NULL));
				goto __catch5_g_error;
				goto __finally5;
			}
			_tmp4_ = NULL;
			if (!((info = (_tmp4_ = _tmp3_, (info == NULL) ? NULL : (info = (g_object_unref (info), NULL)), _tmp4_)) != NULL)) {
				break;
			}
			node = rss_node_new ();
			_tmp5_ = NULL;
			name = (_tmp5_ = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME), (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_));
			rss_node_set_title (node, name);
			_tmp7_ = NULL;
			_tmp6_ = NULL;
			rss_node_set_link (node, _tmp7_ = g_strconcat (_tmp6_ = g_strconcat (base_url, "/", NULL), name, NULL));
			_tmp7_ = (g_free (_tmp7_), NULL);
			_tmp6_ = (g_free (_tmp6_), NULL);
			time = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
			rss_node_set_time (node, (_tmp10_ = (localtime_r ((_tmp9_ = (time_t) time, &_tmp9_), &_tmp8_), _tmp8_), &_tmp10_));
			type = g_file_info_get_file_type (info);
			_tmp11_ = NULL;
			mimetype = (_tmp11_ = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE), (_tmp11_ == NULL) ? NULL : g_strdup (_tmp11_));
			if (type == G_FILE_TYPE_DIRECTORY) {
				char* _tmp13_;
				char* _tmp12_;
				GFile* _tmp14_;
				GFile* dir2;
				GFileEnumerator* files2;
				GFileInfo* info2;
				char* node_description;
				char* _tmp27_;
				_tmp13_ = NULL;
				_tmp12_ = NULL;
				_tmp14_ = NULL;
				dir2 = (_tmp14_ = g_file_new_for_path (_tmp13_ = g_strconcat (_tmp12_ = g_strconcat (path, "/", NULL), name, NULL)), _tmp13_ = (g_free (_tmp13_), NULL), _tmp12_ = (g_free (_tmp12_), NULL), _tmp14_);
				files2 = g_file_enumerate_children (dir2, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME, G_FILE_QUERY_INFO_NONE, NULL, &_inner_error_);
				if (_inner_error_ != NULL) {
					(dir2 == NULL) ? NULL : (dir2 = (g_object_unref (dir2), NULL));
					(node == NULL) ? NULL : (node = (g_object_unref (node), NULL));
					name = (g_free (name), NULL);
					mimetype = (g_free (mimetype), NULL);
					(dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL));
					(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
					(files == NULL) ? NULL : (files = (g_object_unref (files), NULL));
					goto __catch5_g_error;
					goto __finally5;
				}
				info2 = NULL;
				node_description = g_strdup ("<ul>\n");
				while (TRUE) {
					GFileInfo* _tmp15_;
					GFileInfo* _tmp16_;
					const char* _tmp17_;
					char* name2;
					char* _tmp20_;
					char* _tmp19_;
					char* _tmp18_;
					char* _tmp21_;
					char* link2;
					char* _tmp26_;
					char* _tmp25_;
					char* _tmp24_;
					char* _tmp23_;
					char* _tmp22_;
					_tmp15_ = g_file_enumerator_next_file (files2, NULL, &_inner_error_);
					if (_inner_error_ != NULL) {
						(dir2 == NULL) ? NULL : (dir2 = (g_object_unref (dir2), NULL));
						(files2 == NULL) ? NULL : (files2 = (g_object_unref (files2), NULL));
						(info2 == NULL) ? NULL : (info2 = (g_object_unref (info2), NULL));
						node_description = (g_free (node_description), NULL);
						(node == NULL) ? NULL : (node = (g_object_unref (node), NULL));
						name = (g_free (name), NULL);
						mimetype = (g_free (mimetype), NULL);
						(dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL));
						(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
						(files == NULL) ? NULL : (files = (g_object_unref (files), NULL));
						goto __catch5_g_error;
						goto __finally5;
					}
					_tmp16_ = NULL;
					if (!((info2 = (_tmp16_ = _tmp15_, (info2 == NULL) ? NULL : (info2 = (g_object_unref (info2), NULL)), _tmp16_)) != NULL)) {
						break;
					}
					_tmp17_ = NULL;
					name2 = (_tmp17_ = g_file_info_get_attribute_string (info2, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME), (_tmp17_ == NULL) ? NULL : g_strdup (_tmp17_));
					_tmp20_ = NULL;
					_tmp19_ = NULL;
					_tmp18_ = NULL;
					_tmp21_ = NULL;
					link2 = (_tmp21_ = g_strconcat (_tmp20_ = g_strconcat (_tmp19_ = g_strconcat (_tmp18_ = g_strconcat (base_url, "/", NULL), name, NULL), "/", NULL), name2, NULL), _tmp20_ = (g_free (_tmp20_), NULL), _tmp19_ = (g_free (_tmp19_), NULL), _tmp18_ = (g_free (_tmp18_), NULL), _tmp21_);
					_tmp26_ = NULL;
					_tmp25_ = NULL;
					_tmp24_ = NULL;
					_tmp23_ = NULL;
					_tmp22_ = NULL;
					node_description = (_tmp26_ = g_strconcat (node_description, _tmp25_ = g_strconcat (_tmp24_ = g_strconcat (_tmp23_ = g_strconcat (_tmp22_ = g_strconcat ("<li><a href=\"", link2, NULL), "\">", NULL), name2, NULL), "</a></li>\n", NULL), NULL), node_description = (g_free (node_description), NULL), _tmp26_);
					_tmp25_ = (g_free (_tmp25_), NULL);
					_tmp24_ = (g_free (_tmp24_), NULL);
					_tmp23_ = (g_free (_tmp23_), NULL);
					_tmp22_ = (g_free (_tmp22_), NULL);
					name2 = (g_free (name2), NULL);
					link2 = (g_free (link2), NULL);
				}
				_tmp27_ = NULL;
				node_description = (_tmp27_ = g_strconcat (node_description, "<ul>\n", NULL), node_description = (g_free (node_description), NULL), _tmp27_);
				rss_node_set_description (node, node_description);
				(dir2 == NULL) ? NULL : (dir2 = (g_object_unref (dir2), NULL));
				(files2 == NULL) ? NULL : (files2 = (g_object_unref (files2), NULL));
				(info2 == NULL) ? NULL : (info2 = (g_object_unref (info2), NULL));
				node_description = (g_free (node_description), NULL);
			} else {
				if (mimetype == NULL) {
					char* _tmp28_;
					_tmp28_ = NULL;
					mimetype = (_tmp28_ = g_strdup ("application/x-octet-stream"), mimetype = (g_free (mimetype), NULL), _tmp28_);
				}
				if (g_str_has_prefix (mimetype, "image/")) {
					char* _tmp29_;
					_tmp29_ = NULL;
					rss_node_set_description (node, _tmp29_ = g_strdup_printf ("<img src=\"%s\" width=\"300\" />", rss_node_get_link (node)));
					_tmp29_ = (g_free (_tmp29_), NULL);
				} else {
					if (_vala_strcmp0 (mimetype, "text/plain") == 0) {
						char* _tmp31_;
						char* _tmp30_;
						GMappedFile* _tmp32_;
						GMappedFile* f;
						char* _tmp33_;
						_tmp31_ = NULL;
						_tmp30_ = NULL;
						_tmp32_ = NULL;
						f = (_tmp32_ = g_mapped_file_new (_tmp31_ = g_strconcat (_tmp30_ = g_strconcat (path, "/", NULL), name, NULL), FALSE, &_inner_error_), _tmp31_ = (g_free (_tmp31_), NULL), _tmp30_ = (g_free (_tmp30_), NULL), _tmp32_);
						if (_inner_error_ != NULL) {
							(node == NULL) ? NULL : (node = (g_object_unref (node), NULL));
							name = (g_free (name), NULL);
							mimetype = (g_free (mimetype), NULL);
							(dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL));
							(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
							(files == NULL) ? NULL : (files = (g_object_unref (files), NULL));
							goto __catch5_g_error;
							goto __finally5;
						}
						_tmp33_ = NULL;
						rss_node_set_description (node, _tmp33_ = g_strdup_printf ("<pre>%s</pre>\n", (const char*) g_mapped_file_get_contents (f)));
						_tmp33_ = (g_free (_tmp33_), NULL);
						(f == NULL) ? NULL : (f = (g_mapped_file_free (f), NULL));
					} else {
						if (_vala_strcmp0 (mimetype, "text/html") == 0) {
							char* _tmp35_;
							char* _tmp34_;
							GMappedFile* _tmp36_;
							GMappedFile* f;
							char* _tmp37_;
							_tmp35_ = NULL;
							_tmp34_ = NULL;
							_tmp36_ = NULL;
							f = (_tmp36_ = g_mapped_file_new (_tmp35_ = g_strconcat (_tmp34_ = g_strconcat (path, "/", NULL), name, NULL), FALSE, &_inner_error_), _tmp35_ = (g_free (_tmp35_), NULL), _tmp34_ = (g_free (_tmp34_), NULL), _tmp36_);
							if (_inner_error_ != NULL) {
								(node == NULL) ? NULL : (node = (g_object_unref (node), NULL));
								name = (g_free (name), NULL);
								mimetype = (g_free (mimetype), NULL);
								(dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL));
								(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
								(files == NULL) ? NULL : (files = (g_object_unref (files), NULL));
								goto __catch5_g_error;
								goto __finally5;
							}
							_tmp37_ = NULL;
							rss_node_set_description (node, _tmp37_ = g_strdup_printf ("%s\n", (const char*) g_mapped_file_get_contents (f)));
							_tmp37_ = (g_free (_tmp37_), NULL);
							(f == NULL) ? NULL : (f = (g_mapped_file_free (f), NULL));
						} else {
							char* _tmp38_;
							_tmp38_ = NULL;
							rss_node_set_description (node, _tmp38_ = g_strdup_printf (_ ("<a href=\"%s\">Download %s</a>\n"), rss_node_get_link (node), rss_node_get_title (node)));
							_tmp38_ = (g_free (_tmp38_), NULL);
						}
					}
				}
			}
			rss_node_set_content (node, rss_node_get_description (node));
			_tmp39_ = NULL;
			rss->nodes = g_list_append (rss->nodes, (_tmp39_ = node, (_tmp39_ == NULL) ? NULL : g_object_ref (_tmp39_)));
			(node == NULL) ? NULL : (node = (g_object_unref (node), NULL));
			name = (g_free (name), NULL);
			mimetype = (g_free (mimetype), NULL);
		}
		(dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL));
		(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
		(files == NULL) ? NULL : (files = (g_object_unref (files), NULL));
	}
	goto __finally5;
	__catch5_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			fprintf (stderr, _ ("RssFeed.new_from_directory(): Error\n"));
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally5:
	if (_inner_error_ != NULL) {
		(rss == NULL) ? NULL : (rss = (g_object_unref (rss), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	result = rss;
	return result;
}


const char* rss_feed_get_title (RssFeed* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_title;
	return result;
}


void rss_feed_set_title (RssFeed* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_title = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_title = (g_free (self->priv->_title), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "title");
}


const char* rss_feed_get_link (RssFeed* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_link;
	return result;
}


void rss_feed_set_link (RssFeed* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_link = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_link = (g_free (self->priv->_link), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "link");
}


const char* rss_feed_get_description (RssFeed* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_description;
	return result;
}


void rss_feed_set_description (RssFeed* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_description = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_description = (g_free (self->priv->_description), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "description");
}


void rss_feed_get_time (RssFeed* self, struct tm* value) {
	struct tm result;
	g_return_if_fail (self != NULL);
	result = self->priv->_time;
	*value = result;
	return;
}


void rss_feed_set_time (RssFeed* self, struct tm* value) {
	g_return_if_fail (self != NULL);
	self->priv->_time = (*value);
}


const char* rss_feed_get_generator (RssFeed* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_generator;
	return result;
}


void rss_feed_set_generator (RssFeed* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_generator = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_generator = (g_free (self->priv->_generator), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "generator");
}


const char* rss_feed_get_language (RssFeed* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_language;
	return result;
}


void rss_feed_set_language (RssFeed* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_language = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_language = (g_free (self->priv->_language), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "language");
}


static void rss_feed_class_init (RssFeedClass * klass) {
	rss_feed_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (RssFeedPrivate));
	G_OBJECT_CLASS (klass)->get_property = rss_feed_get_property;
	G_OBJECT_CLASS (klass)->set_property = rss_feed_set_property;
	G_OBJECT_CLASS (klass)->finalize = rss_feed_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), RSS_FEED_TITLE, g_param_spec_string ("title", "title", "title", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), RSS_FEED_LINK, g_param_spec_string ("link", "link", "link", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), RSS_FEED_DESCRIPTION, g_param_spec_string ("description", "description", "description", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), RSS_FEED_GENERATOR, g_param_spec_string ("generator", "generator", "generator", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), RSS_FEED_LANGUAGE, g_param_spec_string ("language", "language", "language", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void rss_feed_instance_init (RssFeed * self) {
	const char* _tmp3_;
	const char* _tmp4_;
	const char* _tmp5_;
	self->priv = RSS_FEED_GET_PRIVATE (self);
	self->priv->_title = (_tmp3_ = _ ("No title"), (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_));
	_tmp3_ = NULL;
	self->priv->_link = g_strdup ("");
	self->priv->_description = (_tmp4_ = _ ("No description"), (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_));
	_tmp4_ = NULL;
	self->priv->_generator = g_strdup_printf ("%s/%s", PACKAGE, VERSION);
	self->priv->_language = (_tmp5_ = _ ("en"), (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_));
	_tmp5_ = NULL;
}


static void rss_feed_finalize (GObject* obj) {
	RssFeed * self;
	self = RSS_FEED (obj);
	self->priv->_title = (g_free (self->priv->_title), NULL);
	self->priv->_link = (g_free (self->priv->_link), NULL);
	self->priv->_description = (g_free (self->priv->_description), NULL);
	self->priv->_generator = (g_free (self->priv->_generator), NULL);
	self->priv->_language = (g_free (self->priv->_language), NULL);
	(self->nodes == NULL) ? NULL : (self->nodes = (_g_list_free_g_object_unref (self->nodes), NULL));
	G_OBJECT_CLASS (rss_feed_parent_class)->finalize (obj);
}


GType rss_feed_get_type (void) {
	static GType rss_feed_type_id = 0;
	if (rss_feed_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (RssFeedClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) rss_feed_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (RssFeed), 0, (GInstanceInitFunc) rss_feed_instance_init, NULL };
		rss_feed_type_id = g_type_register_static (G_TYPE_OBJECT, "RssFeed", &g_define_type_info, 0);
	}
	return rss_feed_type_id;
}


static void rss_feed_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	RssFeed * self;
	gpointer boxed;
	self = RSS_FEED (object);
	switch (property_id) {
		case RSS_FEED_TITLE:
		g_value_set_string (value, rss_feed_get_title (self));
		break;
		case RSS_FEED_LINK:
		g_value_set_string (value, rss_feed_get_link (self));
		break;
		case RSS_FEED_DESCRIPTION:
		g_value_set_string (value, rss_feed_get_description (self));
		break;
		case RSS_FEED_GENERATOR:
		g_value_set_string (value, rss_feed_get_generator (self));
		break;
		case RSS_FEED_LANGUAGE:
		g_value_set_string (value, rss_feed_get_language (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void rss_feed_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	RssFeed * self;
	self = RSS_FEED (object);
	switch (property_id) {
		case RSS_FEED_TITLE:
		rss_feed_set_title (self, g_value_get_string (value));
		break;
		case RSS_FEED_LINK:
		rss_feed_set_link (self, g_value_get_string (value));
		break;
		case RSS_FEED_DESCRIPTION:
		rss_feed_set_description (self, g_value_get_string (value));
		break;
		case RSS_FEED_GENERATOR:
		rss_feed_set_generator (self, g_value_get_string (value));
		break;
		case RSS_FEED_LANGUAGE:
		rss_feed_set_language (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




